from typing import List

from qtpy import QtMultimedia

from prettyqt import multimedia
from prettyqt.utils import bidict, InvalidParamError


CAPTURE_MODES = bidict(
    viewfinder=QtMultimedia.QCamera.CaptureViewfinder,
    still_image=QtMultimedia.QCamera.CaptureStillImage,
    video=QtMultimedia.QCamera.CaptureVideo,
)

ERRORS = bidict(
    none=QtMultimedia.QCamera.NoError,
    camera=QtMultimedia.QCamera.CameraError,
    invalid_request=QtMultimedia.QCamera.InvalidRequestError,
    service_missing=QtMultimedia.QCamera.ServiceMissingError,
    not_supported_feature=QtMultimedia.QCamera.NotSupportedFeatureError,
)

LOCK_CHANGE_REASONS = bidict(
    user_request=QtMultimedia.QCamera.UserRequest,
    lock_acquired=QtMultimedia.QCamera.LockAcquired,
    lock_failed=QtMultimedia.QCamera.LockFailed,
    lock_lost=QtMultimedia.QCamera.LockLost,
    lock_temporary_lost=QtMultimedia.QCamera.LockTemporaryLost,
)

LOCK_STATUS = bidict(
    unlocked=QtMultimedia.QCamera.Unlocked,
    searching=QtMultimedia.QCamera.Searching,
    locked=QtMultimedia.QCamera.Locked,
)

LOCK_TYPES = bidict(
    none=QtMultimedia.QCamera.NoLock,
    exposure=QtMultimedia.QCamera.LockExposure,
    white_balance=QtMultimedia.QCamera.LockWhiteBalance,
    focus=QtMultimedia.QCamera.LockFocus,
)

POSITIONS = bidict(
    unspecified=QtMultimedia.QCamera.UnspecifiedPosition,
    back_face=QtMultimedia.QCamera.BackFace,
    front_face=QtMultimedia.QCamera.FrontFace,
)

STATES = bidict(
    unloaded=QtMultimedia.QCamera.UnloadedState,
    loaded=QtMultimedia.QCamera.LoadedState,
    active=QtMultimedia.QCamera.ActiveState,
)

STATUS = bidict(
    active=QtMultimedia.QCamera.ActiveStatus,
    starting=QtMultimedia.QCamera.StartingStatus,
    stopping=QtMultimedia.QCamera.StoppingStatus,
    standby=QtMultimedia.QCamera.StandbyStatus,
    loaded=QtMultimedia.QCamera.LoadedStatus,
    loading=QtMultimedia.QCamera.LoadingStatus,
    unloading=QtMultimedia.QCamera.UnloadingStatus,
    unloaded=QtMultimedia.QCamera.UnloadedStatus,
    unavailable=QtMultimedia.QCamera.UnavailableStatus,
)

QtMultimedia.QCamera.__bases__ = (multimedia.MediaObject,)


class Camera(QtMultimedia.QCamera):
    def get_state(self) -> str:
        """Return current state.

        Possible values: "unloaded", "loaded", "active"

        Returns:
            state
        """
        return STATES.inverse[self.state()]

    def get_status(self) -> str:
        """Return current status.

        Possible values: "active", "starting", "stopping", "standby", "loaded",
                         "loading", "unloading", "unloaded", "unavailable"

        Returns:
            status
        """
        return STATUS.inverse[self.status()]

    def get_lock_status(self) -> str:
        """Return current lock status.

        Possible values: "unlocked", "searching", "locked"

        Returns:
            lock status
        """
        return LOCK_STATUS.inverse[self.lockStatus()]

    def get_error(self) -> str:
        """Return current error state.

        Possible values: "none", "camera", "invalid_request", "service_missing",
                         "not_supported_feature"

        Returns:
            error state
        """
        return ERRORS.inverse[self.error()]

    def set_capture_mode(self, position: str):
        """Set the capture mode.

        Allowed values are "viewfinder", "still_image", "video"

        Args:
            position: capture mode

        Raises:
            InvalidParamError: capture mode does not exist
        """
        if position not in CAPTURE_MODES:
            raise InvalidParamError(position, CAPTURE_MODES)
        self.setCaptureMode(CAPTURE_MODES[position])

    def get_capture_mode(self) -> str:
        """Return current capture mode.

        Possible values: "viewfinder", "still_image", "video"

        Returns:
            capture mode
        """
        return CAPTURE_MODES.inverse[self.captureMode()]

    def get_supported_locks(self) -> List[str]:
        return [k for k, v in LOCK_TYPES.items() if v & self.supportedLocks()]

    def get_requested_locks(self) -> List[str]:
        return [k for k, v in LOCK_TYPES.items() if v & self.requestedLocks()]

    def get_focus(self) -> multimedia.CameraFocus:
        return multimedia.CameraFocus(self.focus())

    def get_exposure(self) -> multimedia.CameraExposure:
        return multimedia.CameraExposure(self.exposure())

    def get_image_processing(self) -> multimedia.CameraImageProcessing:
        return multimedia.CameraImageProcessing(self.imageProcessing())



from qtpy import QtGui, QtCore

from prettyqt import core
from prettyqt.utils import bidict

ACTION = bidict(
    click=QtGui.QInputMethod.Click, context_menu=QtGui.QInputMethod.ContextMenu
)

LAYOUT_DIRECTION = bidict(
    left_to_right=QtCore.Qt.LeftToRight,
    right_to_left=QtCore.Qt.RightToLeft,
    auto=QtCore.Qt.LayoutDirectionAuto,
)

QtGui.QInputMethod.__bases__ = (core.Object,)


class InputMethod:
    def __init__(self, item: QtGui.QInputMethod):
        self.item = item

    def __getattr__(self, val):
        return getattr(self.item, val)

    def get_anchor_rectangle(self) -> core.RectF:
        return core.RectF(self.anchorRectangle())

    def get_cursor_rectangle(self) -> core.RectF:
        return core.RectF(self.cursorRectangle())

    def get_input_item_clip_rectangle(self) -> core.RectF:
        return core.RectF(self.inputItemClipRectangle())

    def get_input_item_rectangle(self) -> core.RectF:
        return core.RectF(self.inputItemRectangle())

    def get_keyboard_rectangle(self) -> core.RectF:
        return core.RectF(self.keyboardRectangle())

    def get_locale(self) -> core.Locale:
        return core.Locale(self.locale())

    def get_input_direction(self) -> str:
        return LAYOUT_DIRECTION.inverse[self.inputDirection()]


if __name__ == "__main__":
    from prettyqt import gui

    app = gui.app()
    method = app.get_input_method()



from typing import Optional

from qtpy import QtWidgets, QtCore

from prettyqt import gui, constants
from prettyqt.utils import InvalidParamError


class TableWidgetItem(QtWidgets.QTableWidgetItem):
    def __setitem__(self, index: int, value):
        self.setData(index, value)

    def __getitem__(self, index: int):
        return self.data(index)

    def serialize_fields(self):
        return dict(
            text=self.text(),
            tool_tip=self.toolTip(),
            status_tip=self.statusTip(),
            checkstate=self.get_checkstate(),
            icon=gui.Icon(self.icon()) if not self.icon().isNull() else None,
            data=self.data(QtCore.Qt.UserRole),
        )

    def set_icon(self, icon: gui.icon.IconType):
        """Set the icon for the action.

        Args:
            icon: icon to use
        """
        icon = gui.icon.get_icon(icon)
        self.setIcon(icon)

    def set_checkstate(self, state: constants.StateStr):
        """Set checkstate of the checkbox.

        Args:
            state: checkstate to use

        Raises:
            InvalidParamError: invalid checkstate
        """
        if state not in constants.STATE:
            raise InvalidParamError(state, constants.STATE)
        self.setCheckState(constants.STATE[state])

    def get_checkstate(self) -> constants.StateStr:
        """Return checkstate.

        Returns:
            checkstate
        """
        return constants.STATE.inverse[self.checkState()]

    def set_text_alignment(
        self,
        horizontal: Optional[constants.HorizontalAlignmentStr] = None,
        vertical: Optional[constants.VerticalAlignmentStr] = None,
    ):
        """Set text alignment of the checkbox.

        Args:
            alignment: text alignment to use

        Raises:
            InvalidParamError: invalid text alignment
        """
        if horizontal is None and vertical is not None:
            flag = constants.V_ALIGNMENT[vertical]
        elif vertical is None and horizontal is not None:
            flag = constants.H_ALIGNMENT[horizontal]
        elif vertical is not None and horizontal is not None:
            flag = constants.V_ALIGNMENT[vertical] | constants.H_ALIGNMENT[horizontal]
        else:
            return
        self.setTextAlignment(flag)

    def get_background(self) -> gui.Brush:
        return gui.Brush(self.background())

    def get_foreground(self) -> gui.Brush:
        return gui.Brush(self.foreground())

    def get_font(self) -> gui.Font:
        return gui.Font(self.font())

    def get_icon(self) -> gui.Icon:
        return gui.Icon(self.icon())


if __name__ == "__main__":
    item = TableWidgetItem()



from typing import Literal

from qtpy import QtCore

from prettyqt.utils import bidict, InvalidParamError


TYPE = bidict(
    precise=QtCore.Qt.PreciseTimer,
    coarse=QtCore.Qt.CoarseTimer,
    very_coarse=QtCore.Qt.VeryCoarseTimer,
)

TypeStr = Literal["precise", "coarse", "very_coarse"]


class BasicTimer(QtCore.QBasicTimer):
    def __bool__(self):
        return self.isActive()

    def start_timer(self, msec: int, obj: QtCore.QObject, timer_type: TypeStr):
        if timer_type not in TYPE:
            raise InvalidParamError(timer_type, TYPE)
        self.start(msec, TYPE[timer_type], obj)



from typing import Union, Iterator
import pathlib

from qtpy import QtGui, QtCore

from prettyqt import core, gui
from prettyqt.utils import bidict, InvalidParamError

if core.VersionNumber.get_qt_version() >= (5, 14, 0):
    MARKDOWN_FEATURES = bidict(
        no_html=QtGui.QTextDocument.MarkdownNoHTML,
        commonmark=QtGui.QTextDocument.MarkdownDialectCommonMark,
        github=QtGui.QTextDocument.MarkdownDialectGitHub,
    )

RESOURCE_TYPES = bidict(
    unknown=QtGui.QTextDocument.UnknownResource,
    html=QtGui.QTextDocument.HtmlResource,
    image=QtGui.QTextDocument.ImageResource,
    stylesheet=QtGui.QTextDocument.StyleSheetResource,
    markdown=QtGui.QTextDocument.MarkdownResource,
    user=QtGui.QTextDocument.UserResource,
)

STACKS = bidict(
    undo=QtGui.QTextDocument.UndoStack,
    redo=QtGui.QTextDocument.RedoStack,
    undo_and_redo=QtGui.QTextDocument.UndoAndRedoStacks,
)

CURSOR_MOVE_STYLES = bidict(
    logical=QtCore.Qt.LogicalMoveStyle, visual=QtCore.Qt.VisualMoveStyle
)

QtGui.QTextDocument.__bases__ = (core.Object,)


class TextDocument(QtGui.QTextDocument):
    def __getitem__(self, index: int) -> gui.TextBlock:
        return gui.TextBlock(self.findBlockByNumber(index))

    def __len__(self) -> int:
        return self.blockCount()

    def __iter__(self) -> Iterator[gui.TextBlock]:
        return iter(self[i] for i in range(self.blockCount()))

    def set_text(self, text: str):
        self.setPlainText(text)

    def serialize_fields(self):
        return dict(
            base_url=core.Url(self.baseUrl()),
            default_font=gui.Font(self.defaultFont()),
            default_stylesheet=self.defaultStyleSheet(),
            default_text_option=gui.TextOption(self.defaultTextOption()),
            document_margin=self.documentMargin(),
            maximum_block_count=self.maximumBlockCount(),
            is_modified=self.isModified(),
            page_size=self.pageSize(),
            text_width=self.textWidth(),
            indent_width=self.indentWidth(),
            undo_redo_enabled=self.isUndoRedoEnabled(),
            use_design_metrics=self.useDesignMetrics(),
        )

    def clear_stacks(self, stack: str):
        """Clear undo / redo stack.

        Allowed values are "undo", "redo", "undo_and_redo"

        Args:
            stack: stack to clear

        Raises:
            InvalidParamError: stack type does not exist
        """
        if stack not in STACKS:
            raise InvalidParamError(stack, STACKS)
        self.clearUndoRedoStacks(STACKS[stack])

    def set_default_cursor_move_style(self, style: str):
        """Set the cursor move style.

        Allowed values are "logical", "visual"

        Args:
            style: cursor move style

        Raises:
            InvalidParamError: cursor move style does not exist
        """
        if style not in CURSOR_MOVE_STYLES:
            raise InvalidParamError(style, CURSOR_MOVE_STYLES)
        self.setDefaultCursorMoveStyle(CURSOR_MOVE_STYLES[style])

    def get_default_cursor_move_style(self) -> str:
        """Return current cursor move style.

        Possible values: "logical", "visual"

        Returns:
            cursor move style
        """
        return CURSOR_MOVE_STYLES.inverse[self.defaultCursorMoveStyle()]

    def add_resource(self, resource_type: str, name: Union[str, pathlib.Path], resource):
        if resource_type not in RESOURCE_TYPES:
            raise InvalidParamError(resource_type, RESOURCE_TYPES)
        url = core.Url(name)
        self.addResource(RESOURCE_TYPES[resource_type], url, resource)


if __name__ == "__main__":
    doc = TextDocument("This is a test\nHello")
    for i in doc:
        print(i)



# This file is based on the original C++ qabstractitemmodeltester.cpp from:
# http://code.qt.io/cgit/qt/qtbase.git/tree/src/testlib/qabstractitemmodeltester.cpp
# Commit 4af292fe5158c2d19e8ab1351c71c3940c7f1032
#
# Licensed under the following terms:
#
# Copyright (C) 2016 The Qt Company Ltd.
# Copyright (C) 2017 Klarälvdalens Datakonsult AB, a KDAB Group company,
# info@kdab.com, author Giuseppe D'Angelo <giuseppe.dangelo@kdab.com>
# Contact: https://www.qt.io/licensing/
#
# This file is part of the QtTest module of the Qt Toolkit.
#
# $QT_BEGIN_LICENSE:LGPL$
# Commercial License Usage
# Licensees holding valid commercial Qt licenses may use this file in
# accordance with the commercial license agreement provided with the
# Software or, alternatively, in accordance with the terms contained in
# a written agreement between you and The Qt Company. For licensing terms
# and conditions see https://www.qt.io/terms-conditions. For further
# information use the contact form at https://www.qt.io/contact-us.
#
# GNU Lesser General Public License Usage
# Alternatively, this file may be used under the terms of the GNU Lesser
# General Public License version 3 as published by the Free Software
# Foundation and appearing in the file LICENSE.LGPL3 included in the
# packaging of this file. Please review the following information to
# ensure the GNU Lesser General Public License version 3 requirements
# will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
#
# GNU General Public License Usage
# Alternatively, this file may be used under the terms of the GNU
# General Public License version 2.0 or (at your option) the GNU General
# Public license version 3 or any later version approved by the KDE Free
# Qt Foundation. The licenses are as published by the Free Software
# Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
# included in the packaging of this file. Please review the following
# information to ensure the GNU General Public License requirements will
# be met: https://www.gnu.org/licenses/gpl-2.0.html and
# https://www.gnu.org/licenses/gpl-3.0.html.
#
# $QT_END_LICENSE$

from typing import List, NamedTuple, Any

from qtpy import QtTest, QtCore, QtGui


class _Changing(NamedTuple):
    parent: QtCore.QModelIndex
    old_size: int
    last: Any
    next: Any


HAS_QT_TESTER = hasattr(QtTest, "QAbstractItemModelTester")

TYPES = [
    (QtCore.Qt.DisplayRole, (str,)),
    (QtCore.Qt.ToolTipRole, (str,)),
    (QtCore.Qt.StatusTipRole, (str,)),
    (QtCore.Qt.WhatsThisRole, (str,)),
    (QtCore.Qt.SizeHintRole, QtCore.QSize),
    (QtCore.Qt.FontRole, QtGui.QFont),
    (QtCore.Qt.BackgroundColorRole, (QtGui.QColor, QtGui.QBrush)),
    (QtCore.Qt.TextColorRole, (QtGui.QColor, QtGui.QBrush)),
    (
        QtCore.Qt.DecorationRole,
        (QtGui.QPixmap, QtGui.QImage, QtGui.QIcon, QtGui.QColor, QtGui.QBrush),
    ),
]


class ModelTester:
    """A tester for Qt's QAbstractItemModels."""

    def __init__(self, model: QtCore.QAbstractItemModel):
        self._model = model
        self._fetching_more = False
        self._insert: List[_Changing] = []
        self._remove: List[_Changing] = []
        self._changing: List[QtCore.QPersistentModelIndex] = []
        self._qt_tester = None

    def _debug(self, text):
        print("modeltest: " + text)

    def _modelindex_debug(self, index):
        """Get a string for debug output for a QModelIndex."""
        if index is None:
            return "<None>"
        elif not index.isValid():
            return "<invalid> (0x{:x})".format(id(index))
        else:
            data = self._model.data(index, QtCore.Qt.DisplayRole)
            return "{}/{} {!r} (0x{:x})".format(
                index.row(), index.column(), data, id(index)
            )

    def check(self, force_py: bool = False):
        """Run a series of checks in the given model.

        Connect to all of the models signals.

        Whenever anything happens recheck everything.

        :param model: The ``QAbstractItemModel`` to test.
        :param force_py:
          Force using the Python implementation, even if the C++ implementation
          is available.
        """
        assert self._model is not None

        if HAS_QT_TESTER and not force_py:
            reporting_mode = QtTest.QAbstractItemModelTester.FailureReportingMode.Warning
            self._qt_tester = QtTest.QAbstractItemModelTester(self._model, reporting_mode)
            self._debug("Using Qt C++ tester")
            return

        self._debug("Using Python tester")

        self._fetching_more = False
        self._insert = []
        self._remove = []
        self._changing = []

        self._model.columnsAboutToBeInserted.connect(self._run)
        self._model.columnsAboutToBeRemoved.connect(self._run)
        self._model.columnsInserted.connect(self._run)
        self._model.columnsRemoved.connect(self._run)
        self._model.dataChanged.connect(self._run)
        self._model.headerDataChanged.connect(self._run)
        self._model.layoutAboutToBeChanged.connect(self._run)
        self._model.layoutChanged.connect(self._run)
        self._model.modelReset.connect(self._run)
        self._model.rowsAboutToBeInserted.connect(self._run)
        self._model.rowsAboutToBeRemoved.connect(self._run)
        self._model.rowsInserted.connect(self._run)
        self._model.rowsRemoved.connect(self._run)

        # Special checks for changes
        self._model.layoutAboutToBeChanged.connect(self._on_layout_about_to_be_changed)
        self._model.layoutChanged.connect(self._on_layout_changed)
        self._model.rowsAboutToBeInserted.connect(self._on_rows_about_to_be_inserted)
        self._model.rowsAboutToBeRemoved.connect(self._on_rows_about_to_be_removed)
        self._model.rowsInserted.connect(self._on_rows_inserted)
        self._model.rowsRemoved.connect(self._on_rows_removed)
        self._model.dataChanged.connect(self._on_data_changed)
        self._model.headerDataChanged.connect(self._on_header_data_changed)

        self._run()

    def _cleanup(self):
        """Not API intended for users, but called from the fixture function."""
        if self._model is None:
            return

        self._model.columnsAboutToBeInserted.disconnect(self._run)
        self._model.columnsAboutToBeRemoved.disconnect(self._run)
        self._model.columnsInserted.disconnect(self._run)
        self._model.columnsRemoved.disconnect(self._run)
        self._model.dataChanged.disconnect(self._run)
        self._model.headerDataChanged.disconnect(self._run)
        self._model.layoutAboutToBeChanged.disconnect(self._run)
        self._model.layoutChanged.disconnect(self._run)
        self._model.modelReset.disconnect(self._run)
        self._model.rowsAboutToBeInserted.disconnect(self._run)
        self._model.rowsAboutToBeRemoved.disconnect(self._run)
        self._model.rowsInserted.disconnect(self._run)
        self._model.rowsRemoved.disconnect(self._run)

        self._model.layoutAboutToBeChanged.disconnect(self._on_layout_about_to_be_changed)
        self._model.layoutChanged.disconnect(self._on_layout_changed)
        self._model.rowsAboutToBeInserted.disconnect(self._on_rows_about_to_be_inserted)
        self._model.rowsAboutToBeRemoved.disconnect(self._on_rows_about_to_be_removed)
        self._model.rowsInserted.disconnect(self._on_rows_inserted)
        self._model.rowsRemoved.disconnect(self._on_rows_removed)
        self._model.dataChanged.disconnect(self._on_data_changed)
        self._model.headerDataChanged.disconnect(self._on_header_data_changed)

        self._model = None

    def _run(self):
        assert self._model is not None
        assert self._fetching_more is not None
        if self._fetching_more:
            return
        self._test_basic()
        self._test_row_count_and_column_count()
        self._test_has_index()
        self._test_index()
        self._test_parent()
        self._test_data()

    def _test_basic(self):
        """Try to call a number of the basic functions (not all).

        Make sure the model doesn't outright segfault, testing the functions
        which make sense.
        """
        assert not self._model.buddy(QtCore.QModelIndex()).isValid()

        self._model.canFetchMore(QtCore.QModelIndex())
        assert self._column_count(QtCore.QModelIndex()) >= 0
        self._fetch_more(QtCore.QModelIndex())
        flags = self._model.flags(QtCore.QModelIndex())
        assert flags == QtCore.Qt.ItemIsDropEnabled or not flags
        self._has_children(QtCore.QModelIndex())

        has_row = self._model.hasIndex(0, 0)
        if has_row:
            cache = None
            self._model.match(self._model.index(0, 0), -1, cache)

        self._model.mimeTypes()
        assert not self._parent(QtCore.QModelIndex()).isValid()
        assert self._model.rowCount() >= 0
        self._model.span(QtCore.QModelIndex())

        self._model.supportedDropActions()
        self._model.roleNames()

    def _test_row_count_and_column_count(self):
        """Test model's implementation of row/columnCount() and hasChildren().

        Models that are dynamically populated are not as fully tested here.

        The models rowCount() is tested more extensively in _check_children(),
        but this catches the big mistakes.
        """
        # check top row
        top_index = self._model.index(0, 0, QtCore.QModelIndex())

        rows = self._model.rowCount(top_index)
        assert rows >= 0

        columns = self._column_count(top_index)
        assert columns >= 0

        if rows == 0 or columns == 0:
            return

        assert self._has_children(top_index)

        second_level_index = self._model.index(0, 0, top_index)
        assert second_level_index.isValid()

        rows = self._model.rowCount(second_level_index)
        assert rows >= 0

        columns = self._column_count(second_level_index)
        assert columns >= 0

        if rows == 0 or columns == 0:
            return

        assert self._has_children(second_level_index)

    def _test_has_index(self):
        """Test model's implementation of hasIndex().

        hasIndex() is tested more extensively in _check_children(),
        but this catches the big mistakes.
        """
        # Make sure that invalid values return an invalid index
        assert not self._model.hasIndex(-2, -2)
        assert not self._model.hasIndex(-2, 0)
        assert not self._model.hasIndex(0, -2)

        rows = self._model.rowCount()
        columns = self._column_count()

        # check out of bounds
        assert not self._model.hasIndex(rows, columns)
        assert not self._model.hasIndex(rows + 1, columns + 1)

        if rows > 0 and columns > 0:
            assert self._model.hasIndex(0, 0)

    def _test_index(self):
        """Test model's implementation of index().

        index() is tested more extensively in _check_children(),
        but this catches the big mistakes.
        """
        rows = self._model.rowCount()
        columns = self._column_count()

        for row in range(rows):
            for column in range(columns):
                # Make sure that the same index is *always* returned
                a = self._model.index(row, column)
                b = self._model.index(row, column)
                assert a.isValid()
                assert b.isValid()
                assert a == b

    def _test_parent(self):
        """Test model's implementation of QAbstractItemModel::parent()."""
        # Make sure the model won't crash and will return an invalid
        # QModelIndex when asked for the parent of an invalid index.
        assert not self._parent(QtCore.QModelIndex()).isValid()

        if not self._has_children():
            return

        # Column 0                | Column 1      |
        # QModelIndex()           |               |
        #    \- top_index         | top_index_1   |
        #         \- child_index  | child_index_1 |

        # Common error test #1, make sure that a top level index has a parent
        # that is a invalid QModelIndex.
        top_index = self._model.index(0, 0, QtCore.QModelIndex())
        assert not self._parent(top_index).isValid()

        # Common error test #2, make sure that a second level index has a
        # parent that is the first level index.
        if self._has_children(top_index):
            child_index = self._model.index(0, 0, top_index)
            assert self._parent(child_index) == top_index

        # Common error test #3, the second column should NOT have the same
        # children as the first column in a row.
        # Usually the second column shouldn't have children.
        if self._model.hasIndex(0, 1):
            top_index_1 = self._model.index(0, 1, QtCore.QModelIndex())
            if self._has_children(top_index) and self._has_children(top_index_1):
                child_index = self._model.index(0, 0, top_index)
                assert child_index.isValid()
                child_index_1 = self._model.index(0, 0, top_index_1)
                assert child_index_1.isValid()
                assert child_index != child_index_1

        # Full test, walk n levels deep through the model making sure that all
        # parent's children correctly specify their parent.
        self._check_children(QtCore.QModelIndex())

    def _check_children(self, parent, current_depth=0):
        """Check parent/children relationships.

        Called from the parent() test.

        A model that returns an index of parent X should also return X when
        asking for the parent of the index.

        This recursive function does pretty extensive testing on the whole
        model in an effort to catch edge cases.

        This function assumes that rowCount(), columnCount() and index()
        already work.  If they have a bug it will point it out, but the above
        tests should have already found the basic bugs because it is easier to
        figure out the problem in those tests then this one.
        """
        # First just try walking back up the tree.
        p = parent
        while p.isValid():
            p = p.parent()

        # For models that are dynamically populated
        if self._model.canFetchMore(parent):
            self._fetch_more(parent)

        rows = self._model.rowCount(parent)
        columns = self._column_count(parent)

        if rows > 0:
            assert self._has_children(parent)

        # Some further testing against rows(), columns(), and hasChildren()
        assert rows >= 0
        assert columns >= 0
        if rows > 0 and columns > 0:
            assert self._has_children(parent)
        self._debug(
            "Checking children of {} with depth {} "
            "({} rows, {} columns)".format(
                self._modelindex_debug(parent), current_depth, rows, columns
            )
        )

        top_left_child = self._model.index(0, 0, parent)
        assert not self._model.hasIndex(rows, 0, parent)
        assert not self._model.hasIndex(rows + 1, 0, parent)

        for r in range(rows):
            assert not self._model.hasIndex(r, columns, parent)
            assert not self._model.hasIndex(r, columns + 1, parent)

            for c in range(columns):
                assert self._model.hasIndex(r, c, parent)
                index = self._model.index(r, c, parent)
                # rowCount() and columnCount() said that it existed...
                if not index.isValid():
                    self._debug(
                        "Got invalid index at row={} col={} parent={}".format(
                            r, c, self._modelindex_debug(parent)
                        )
                    )
                assert index.isValid()

                # index() should always return the same index when called twice
                # in a row
                modified_index = self._model.index(r, c, parent)
                assert index == modified_index

                sibling = self._model.sibling(r, c, top_left_child)
                assert index == sibling

                sibling2 = top_left_child.sibling(r, c)
                assert index == sibling2

                # Some basic checking on the index that is returned
                assert index.model() == self._model
                assert index.row() == r
                assert index.column() == c

                # If the next test fails here is some somewhat useful debug you
                # play with.
                if self._parent(index) != parent:
                    self._debug(
                        "Inconsistent parent() implementation detected\n"
                        "  index={} exp. parent={} act. parent={}\n"
                        "  row={} col={} depth={}\n"
                        "  data for child: {}\n"
                        "  data for parent: {}\n".format(
                            self._modelindex_debug(index),
                            self._modelindex_debug(parent),
                            self._modelindex_debug(self._parent(index)),
                            r,
                            c,
                            current_depth,
                            self._model.data(index),
                            self._model.data(parent),
                        )
                    )

                # Check that we can get back our real parent.
                assert self._parent(index) == parent

                # recursively go down the children
                if self._has_children(index) and current_depth < 10:
                    self._debug(
                        "{} has {} children".format(
                            self._modelindex_debug(index), self._model.rowCount(index)
                        )
                    )
                    self._check_children(index, current_depth + 1)

                # make sure that after testing the children that the index
                # doesn't change.
                newer_index = self._model.index(r, c, parent)
                assert index == newer_index
        self._debug("Children check for {} done".format(self._modelindex_debug(parent)))

    def _test_data(self):
        """Test model's implementation of data()."""
        if not self._has_children():
            return

        # A valid index should have a valid QVariant data
        assert self._model.index(0, 0).isValid()
        # General purpose roles with a fixed expected type
        for role, typ in TYPES:
            data = self._model.data(self._model.index(0, 0), role)
            if data is not None:
                data = data
            assert data == None or isinstance(data, typ), role  # noqa

        # Check that the alignment is one we know about
        alignment = self._model.data(self._model.index(0, 0), QtCore.Qt.TextAlignmentRole)
        if alignment is not None:
            try:
                alignment = int(alignment)
            except (TypeError, ValueError):
                assert 0, "%r should be a TextAlignmentRole enum" % alignment
            mask = int(QtCore.Qt.AlignHorizontal_Mask | QtCore.Qt.AlignVertical_Mask)
            assert alignment == alignment & mask

        # Check that the "check state" is one we know about.
        state = self._model.data(self._model.index(0, 0), QtCore.Qt.CheckStateRole)
        assert state in [
            None,
            QtCore.Qt.Unchecked,
            QtCore.Qt.PartiallyChecked,
            QtCore.Qt.Checked,
        ]

    def _on_rows_about_to_be_inserted(
        self, parent: QtCore.QModelIndex, start: int, end: int
    ):
        """Store what is about to be inserted.

        This gets stored to make sure it actually happens in rowsInserted.
        """
        last_index = self._model.index(start - 1, 0, parent)
        next_index = self._model.index(start, 0, parent)
        parent_rowcount = self._model.rowCount(parent)

        self._debug(
            "rows about to be inserted: start {}, end {}, parent {}, "
            "parent row count {}, last item {}, next item {}".format(
                start,
                end,
                self._modelindex_debug(parent),
                parent_rowcount,
                self._modelindex_debug(last_index),
                self._modelindex_debug(next_index),
            )
        )

        last_data = self._model.data(last_index) if start > 0 else None
        next_data = self._model.data(next_index) if start < parent_rowcount else None
        c = _Changing(
            parent=parent, old_size=parent_rowcount, last=last_data, next=next_data
        )
        self._insert.append(c)

    def _on_rows_inserted(self, parent: QtCore.QModelIndex, start: int, end: int):
        """Confirm that what was said was going to happen actually did."""
        c = self._insert.pop()
        last_data = (
            self._model.data(self._model.index(start - 1, 0, parent))
            if start >= 1
            else None
        )
        next_data = (
            self._model.data(self._model.index(end + 1, 0, c.parent))
            if end + 1 < self._model.rowCount(c.parent)
            else None
        )
        expected_size = c.old_size + (end - start + 1)
        current_size = self._model.rowCount(parent)

        self._debug(f"rows inserted: start {start}, end {end}")
        self._debug(
            "  from rowsAboutToBeInserted: parent {}, "
            "size {} (-> {} expected), "
            "next data {!r}, last data {!r}".format(
                self._modelindex_debug(c.parent),
                c.old_size,
                expected_size,
                c.next,
                c.last,
            )
        )

        self._debug(
            "  now in rowsInserted:        parent {}, size {}, "
            "next data {!r}, last data {!r}".format(
                self._modelindex_debug(parent), current_size, next_data, last_data
            )
        )
        assert c.parent == parent

        for ii in range(start, end + 1):
            idx = self._model.index(ii, 0, parent)
            self._debug(" item {} inserted: {}".format(ii, self._modelindex_debug(idx)))
        self._debug("")

        assert current_size == expected_size
        if last_data is not None:
            assert c.last == last_data
        if next_data is not None:
            assert c.next == next_data

    def _on_layout_about_to_be_changed(self):
        for i in range(max(self._model.rowCount(), 100)):
            idx = QtCore.QPersistentModelIndex(self._model.index(i, 0))
            self._changing.append(idx)

    def _on_layout_changed(self):
        for p in self._changing:
            assert p == self._model.index(p.row(), p.column(), p.parent())
        self._changing = []

    def _on_rows_about_to_be_removed(
        self, parent: QtCore.QModelIndex, start: int, end: int
    ):
        """Store what is about to be removed to make sure it actually happens.

        This gets stored to make sure it actually happens in rowsRemoved.
        """
        parent_rowcount = self._model.rowCount(parent)
        last_index = self._model.index(start - 1, 0, parent) if start > 0 else None
        next_index = (
            self._model.index(end + 1, 0, parent) if end < parent_rowcount - 1 else None
        )

        self._debug(
            "rows about to be removed: start {}, end {}, parent {}, "
            "parent row count {}, last item {}, next item {}".format(
                start,
                end,
                self._modelindex_debug(parent),
                parent_rowcount,
                self._modelindex_debug(last_index),
                self._modelindex_debug(next_index),
            )
        )

        if last_index is not None:
            assert last_index.isValid()
        if next_index is not None:
            assert next_index.isValid()

        last_data = None if last_index is None else self._model.data(last_index)
        next_data = None if next_index is None else self._model.data(next_index)
        c = _Changing(
            parent=parent, old_size=parent_rowcount, last=last_data, next=next_data
        )
        self._remove.append(c)

    def _on_rows_removed(self, parent: QtCore.QModelIndex, start: int, end: int):
        """Confirm that what was said was going to happen actually did."""
        c = self._remove.pop()
        last_data = (
            self._model.data(self._model.index(start - 1, 0, c.parent))
            if start > 0
            else None
        )
        next_data = (
            self._model.data(self._model.index(start, 0, c.parent))
            if end < c.old_size - 1
            else None
        )
        current_size = self._model.rowCount(parent)
        expected_size = c.old_size - (end - start + 1)

        self._debug(f"rows removed: start {start}, end {end}")
        self._debug(
            "  from rowsAboutToBeRemoved: parent {}, "
            "size {} (-> {} expected), "
            "next data {!r}, last data {!r}".format(
                self._modelindex_debug(c.parent),
                c.old_size,
                expected_size,
                c.next,
                c.last,
            )
        )

        self._debug(
            "  now in rowsRemoved:        parent {}, size {}, "
            "next data {!r}, last data {!r}".format(
                self._modelindex_debug(parent), current_size, next_data, last_data
            )
        )

        assert c.parent == parent
        assert current_size == expected_size
        if last_data is not None:
            assert c.last == last_data
        if next_data is not None:
            assert c.next == next_data

    def _on_data_changed(
        self, top_left: QtCore.QModelIndex, bottom_right: QtCore.QModelIndex
    ):
        assert top_left.isValid()
        assert bottom_right.isValid()
        common_parent = bottom_right.parent()
        assert top_left.parent() == common_parent
        assert top_left.row() <= bottom_right.row()
        assert top_left.column() <= bottom_right.column()
        assert bottom_right.row() < self._model.rowCount(common_parent)
        assert bottom_right.column() < self._column_count(common_parent)

    def _on_header_data_changed(self, orientation, start: int, end: int):
        assert orientation in [QtCore.Qt.Horizontal, QtCore.Qt.Vertical]
        is_vertical = orientation == QtCore.Qt.Vertical
        count = self._model.rowCount() if is_vertical else self._column_count()
        assert 0 <= start <= end < count

    def _column_count(self, parent: QtCore.QModelIndex = QtCore.QModelIndex()):
        """Test columnCount.

        Workaround for the fact that ``columnCount`` is a private method in
        QAbstractListModel/QAbstractTableModel subclasses.
        """
        if isinstance(self._model, QtCore.QAbstractListModel):
            return 1 if parent == QtCore.QModelIndex() else 0
        else:
            return self._model.columnCount(parent)

    def _parent(self, index: QtCore.QModelIndex):
        """Test parent."""
        model_types = (QtCore.QAbstractListModel, QtCore.QAbstractTableModel)
        if isinstance(self._model, model_types):
            return QtCore.QModelIndex()
        else:
            return self._model.parent(index)

    def _has_children(self, parent: QtCore.QModelIndex = QtCore.QModelIndex()):
        """Test hasChildren."""
        model_types = (QtCore.QAbstractListModel, QtCore.QAbstractTableModel)
        if isinstance(self._model, model_types):
            print(parent == QtCore.QModelIndex(), self._model.rowCount())
            return parent == QtCore.QModelIndex() and self._model.rowCount() > 0
        else:
            return self._model.hasChildren(parent)

    def _fetch_more(self, parent: QtCore.QModelIndex):
        """Call ``fetchMore`` on the model and set ``self._fetching_more``."""
        self._fetching_more = True
        self._model.fetchMore(parent)
        self._fetching_more = False



from typing import Optional

from qtpy import QtGui


class PixmapCache(QtGui.QPixmapCache):
    def __setitem__(self, key: str, value: QtGui.QPixmap):
        self.insert(key, value)

    def __getitem__(self, key: str) -> Optional[QtGui.QPixmap]:
        return self.find(key)

    def __delitem__(self, key: str):
        self.remove(key)


if __name__ == "__main__":
    from prettyqt import gui

    app = gui.app()
    cache = PixmapCache()
    pix = QtGui.QPixmap()
    cache["test"] = pix
    assert cache["test"] == pix



from typing import Union, Iterator, Optional

from qtpy import QtQml, QtCore

from prettyqt import core


QtQml.QQmlApplicationEngine.__bases__ = (QtQml.QQmlEngine,)


class QmlApplicationEngine(QtQml.QQmlApplicationEngine):
    def __iter__(self) -> Iterator[QtCore.QObject]:
        return iter(self.rootObjects())

    def load_data(
        self,
        data: Union[QtCore.QByteArray, bytes, str],
        url: Optional[Union[QtCore.QUrl, str]] = None,
    ):
        if isinstance(data, str):
            data = data.encode()
        if isinstance(data, bytes):
            data = QtCore.QByteArray(data)
        if isinstance(url, str):
            url = core.Url.from_user_input(url)
        elif url is None:
            url = core.Url()
        self.loadData(data, url)



from typing import Literal

from qtpy import QtCore

from prettyqt.utils import bidict, InvalidParamError


TYPE = bidict(
    precise=QtCore.Qt.PreciseTimer,
    coarse=QtCore.Qt.CoarseTimer,
    very_coarse=QtCore.Qt.VeryCoarseTimer,
)

TypeStr = Literal["precise", "coarse", "very_coarse"]


class DeadlineTimer(QtCore.QDeadlineTimer):
    def set_type(self, typ: TypeStr):
        """Set the timer type.

        Allowed values are "precise", "coarse", "very_coarse"

        Args:
            typ: timer type

        Raises:
            InvalidParamError: timer type does not exist
        """
        if typ not in TYPE:
            raise InvalidParamError(typ, TYPE)
        self.setTimerType(TYPE[typ])

    def get_type(self) -> TypeStr:
        """Return current timer type.

        Possible values: "precise", "coarse", "very_coarse"

        Returns:
            timer type
        """
        return TYPE.inverse[self.timerType()]



"""The MIT License (MIT).

Copyright (c) 2012-2014 Alexander Turkin
Copyright (c) 2014 William Hallatt
Copyright (c) 2015 Jacob Dawid
Copyright (c) 2016 Luca Weiss
Copyright (c) 2017 Philipp Temminghoff

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import math
from typing import Optional

from qtpy import QtCore, QtWidgets

from prettyqt import core, gui, widgets
from prettyqt.utils import colors


class BaseWaitingSpinner(widgets.Widget):
    def __init__(
        self,
        parent: Optional[QtWidgets.QWidget],
        modality: widgets.widget.ModalityStr = "none",
    ):
        super().__init__(parent=parent)

        # WAS IN initialize()
        self._color = gui.Color("black")
        self._roundness = 100.0
        self._minimum_trail_opacity = 3.14159265358979323846
        self._trail_fade_percentage = 80.0
        self._revolutions_per_second = 1.57079632679489661923
        self._line_num = 20
        self._line_length = 10
        self._line_width = 2
        self._inner_radius = 10
        self._current_counter = 0
        self._timer = core.Timer(self)
        self._timer.timeout.connect(self._rotate)
        self._update_size()
        self._update_timer()
        self.hide()
        # END initialize()

        self.set_modality(modality)
        # self.setAttribute(QtCore.Qt.WA_TranslucentBackground)

    def paintEvent(self, event):
        painter = gui.Painter(self)
        painter.fill_rect(self.rect(), "transparent")
        painter.use_antialiasing()

        if self._current_counter >= self._line_num:
            self._current_counter = 0

        painter.set_pen(style="none")
        painter.translate(
            self._inner_radius + self._line_length, self._inner_radius + self._line_length
        )
        rect = core.RectF(0, -self._line_width / 2, self._line_length, self._line_width)
        for i in range(self._line_num):
            with painter.backup_state():
                rotate_angle = 360 * i / self._line_num
                painter.rotate(rotate_angle)
                painter.translate(self._inner_radius, 0)
                distance = self.linecount_distance_from_primary(
                    i, self._current_counter, self._line_num
                )
                color = self.current_line_color(
                    distance,
                    self._line_num,
                    self._trail_fade_percentage,
                    self._minimum_trail_opacity,
                    self._color,
                )
                painter.setBrush(color)
                painter.drawRoundedRect(
                    rect, self._roundness, self._roundness, QtCore.Qt.RelativeSize
                )

    def start(self):
        self.show()
        if not self._timer.isActive():
            self._timer.start()
            self._current_counter = 0

    def stop(self):
        self.hide()
        if self._timer.isActive():
            self._timer.stop()
            self._current_counter = 0

    def set_line_num(self, lines: int):
        self._line_num = lines
        self._current_counter = 0
        self._update_timer()

    def set_line_length(self, length: int):
        self._line_length = length
        self._update_size()

    def set_line_width(self, width: int):
        self._line_width = width

    def set_inner_radius(self, radius: int):
        self._inner_radius = radius
        self._update_size()

    def color(self) -> gui.Color:
        return self._color

    def roundness(self) -> float:
        return self._roundness

    def minimum_trail_opacity(self) -> float:
        return self._minimum_trail_opacity

    def trail_fade_percentage(self) -> float:
        return self._trail_fade_percentage

    def revolutions_per_second(self) -> float:
        return self._revolutions_per_second

    def line_num(self) -> int:
        return self._line_num

    def line_length(self) -> int:
        return self._line_length

    def line_width(self) -> int:
        return self._line_width

    def inner_radius(self) -> int:
        return self._inner_radius

    def is_spinning(self) -> bool:
        return self._timer.isActive()

    def set_roundness(self, roundness: float):
        self._roundness = max(0.0, min(100.0, roundness))

    def set_color(self, color: colors.ColorType = "black"):
        self._color = colors.get_color(color)

    def set_revolutions_per_second(self, _revolutions_per_second: float):
        self._revolutions_per_second = _revolutions_per_second
        self._update_timer()

    def set_trail_fade_percentage(self, trail: float):
        self._trail_fade_percentage = trail

    def set_minimum_trail_opacity(self, minimum_trail_opacity: float):
        self._minimum_trail_opacity = minimum_trail_opacity

    def _rotate(self):
        self._current_counter += 1
        if self._current_counter >= self._line_num:
            self._current_counter = 0
        self.update()

    def _update_size(self):
        size = (self._inner_radius + self._line_length) * 2
        self.setFixedSize(size, size)

    def _update_timer(self):
        divider = int(self._line_num * self._revolutions_per_second)
        self._timer.setInterval(1000 // divider)

    def linecount_distance_from_primary(
        self, current: int, primary: int, total_lines: int
    ) -> int:
        distance = primary - current
        if distance < 0:
            distance += total_lines
        return distance

    def current_line_color(
        self,
        count_distance: int,
        total_lines: int,
        fade_perc: float,
        min_opacity: float,
        color: gui.Color,
    ) -> gui.Color:
        color = gui.Color(color)
        if count_distance == 0:
            return color
        min_alpha_f = min_opacity / 100
        dist_threshold = int(math.ceil((total_lines - 1) * fade_perc / 100))
        if count_distance > dist_threshold:
            color.setAlphaF(min_alpha_f)
        else:
            alpha = color.alphaF()
            alpha_diff = alpha - min_alpha_f
            gradient = alpha_diff / (dist_threshold + 1)
            result_alpha = alpha - gradient * count_distance
            # If alpha is out of bounds, clip it.
            result_alpha = min(1.0, max(0.0, result_alpha))
            color.setAlphaF(result_alpha)
        return color


class WaitingSpinner(BaseWaitingSpinner):
    def __init__(
        self,
        parent: Optional[QtWidgets.QWidget],
        center_on_parent: bool = True,
        disable_parent: bool = True,
        modality: widgets.widget.ModalityStr = "none",
        additional_disabled=None,
    ):
        super().__init__(parent=parent, modality=modality)
        self._center_on_parent = center_on_parent
        self._disable_parent = disable_parent
        self.additional_disabled = additional_disabled if additional_disabled else []

    def paintEvent(self, event):
        self._update_position()
        super().paintEvent(event)

    def start(self):
        self._update_position()
        super().start()
        if self.parentWidget and self._disable_parent:
            self.parentWidget().setEnabled(False)
            for item in self.additional_disabled:
                item.setEnabled(False)

    def stop(self):
        super().stop()
        if self.parentWidget() and self._disable_parent:
            self.parentWidget().setEnabled(True)
            for item in self.additional_disabled:
                item.setEnabled(True)

    def _update_position(self):
        parent = self.parentWidget()
        if parent and self._center_on_parent:
            self.move(
                parent.width() // 2 - self.width() // 2,
                parent.height() // 2 - self.height() // 2,
            )


if __name__ == "__main__":
    app = widgets.app()
    mainwindow = widgets.MainWindow()
    spinner = WaitingSpinner(mainwindow)
    mainwindow.show()
    spinner.start()
    app.main_loop()



from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QStyleOptionToolButton.__bases__ = (widgets.StyleOptionComplex,)


class StyleOptionToolButton(QtWidgets.QStyleOptionToolButton):
    pass



from qtpy import PYQT5, PYSIDE2

if PYQT5:
    from PyQt5 import QtPositioning
elif PYSIDE2:
    from PySide2 import QtPositioning


class GeoAddress(QtPositioning.QGeoAddress):
    def __str__(self):
        return self.text()



from qtpy import QtMultimediaWidgets, QtCore

from prettyqt import multimedia, widgets


QtMultimediaWidgets.QVideoWidget.__bases__ = (
    widgets.Widget,
    multimedia.MediaBindableInterface,
)


class VideoWidget(QtMultimediaWidgets.QVideoWidget):
    def __init__(self, *args, **kwargs):
        self.doubleclick_for_fullscreen = False
        super().__init__(*args, **kwargs)

    def set_doubleclick_for_fullscreen(self, value: bool = True):
        self.doubleclick_for_fullscreen = value

    def mouseDoubleClickEvent(self, event):
        if event.button() == QtCore.Qt.LeftButton and self.doubleclick_for_fullscreen:
            self.setFullScreen(not self.isFullScreen())
            event.accept()
        return super().mouseDoubleClickEvent(event)



from typing import List, Iterator, Union

from qtpy import QtNetwork, QtCore

from prettyqt import core


QtNetwork.QNetworkCookieJar.__bases__ = (core.Object,)


class NetworkCookieJar(QtNetwork.QNetworkCookieJar):
    def __add__(self, other: QtNetwork.QNetworkCookie):
        self.insertCookie(other)
        return self

    def __getitem__(self, url: str) -> List[QtNetwork.QNetworkCookie]:
        url = core.Url(url)
        return self.cookiesForUrl(url)

    def __repr__(self):
        return f"{type(self).__name__}()"

    def __iter__(self) -> Iterator[QtNetwork.QNetworkCookie]:
        return iter(self.allCookies())

    def set_cookies_from_url(
        self, cookies: List[QtNetwork.QNetworkCookie], url: Union[QtCore.QUrl, str]
    ) -> bool:
        return self.setCookiesFromUrl(cookies, QtCore.QUrl(url))



from qtpy import QtWidgets

from prettyqt import gui, widgets


QtWidgets.QFontDialog.__bases__ = (widgets.BaseDialog,)


class FontDialog(QtWidgets.QFontDialog):
    def get_current_font(self) -> gui.Font:
        return gui.Font(self.currentFont())


if __name__ == "__main__":
    app = widgets.app()
    widget = FontDialog.getFont()
    app.main_loop()



from qtpy import QtCore

from prettyqt.utils import bidict

Loc = QtCore.QLocale

COUNTRY = bidict(
    any=Loc.AnyCountry,
    afghanistan=Loc.Afghanistan,
    aland_islands=Loc.AlandIslands,
    albania=Loc.Albania,
    algeria=Loc.Algeria,
    american_samoa=Loc.AmericanSamoa,
    andorra=Loc.Andorra,
    angola=Loc.Angola,
    anguilla=Loc.Anguilla,
    antarctica=Loc.Antarctica,
    antigua_and_barbuda=Loc.AntiguaAndBarbuda,
    argentina=Loc.Argentina,
    armenia=Loc.Armenia,
    aruba=Loc.Aruba,
    ascension_island=Loc.AscensionIsland,
    australia=Loc.Australia,
    austria=Loc.Austria,
    azerbaijan=Loc.Azerbaijan,
    bahamas=Loc.Bahamas,
    bahrain=Loc.Bahrain,
    bangladesh=Loc.Bangladesh,
    barbados=Loc.Barbados,
    belarus=Loc.Belarus,
    belgium=Loc.Belgium,
    belize=Loc.Belize,
    benin=Loc.Benin,
    bermuda=Loc.Bermuda,
    bhutan=Loc.Bhutan,
    bolivia=Loc.Bolivia,
    bonaire=Loc.Bonaire,
    bosnia_and_herzegowina=Loc.BosniaAndHerzegowina,
    botswana=Loc.Botswana,
    bouvet_island=Loc.BouvetIsland,
    brazil=Loc.Brazil,
    british_indian_ocean_territory=Loc.BritishIndianOceanTerritory,
    british_virgin_islands=Loc.BritishVirginIslands,
    brunei=Loc.Brunei,
    bulgaria=Loc.Bulgaria,
    burkina_faso=Loc.BurkinaFaso,
    burundi=Loc.Burundi,
    cambodia=Loc.Cambodia,
    cameroon=Loc.Cameroon,
    canada=Loc.Canada,
    canary_islands=Loc.CanaryIslands,
    cape_verde=Loc.CapeVerde,
    cayman_islands=Loc.CaymanIslands,
    central_african_republic=Loc.CentralAfricanRepublic,
    ceuta_and_melilla=Loc.CeutaAndMelilla,
    chad=Loc.Chad,
    chile=Loc.Chile,
    china=Loc.China,
    christmas_island=Loc.ChristmasIsland,
    clipperton_island=Loc.ClippertonIsland,
    cocos_islands=Loc.CocosIslands,
    colombia=Loc.Colombia,
    comoros=Loc.Comoros,
    congo_brazzaville=Loc.CongoBrazzaville,
    congo_kinshasa=Loc.CongoKinshasa,
    cook_islands=Loc.CookIslands,
    costa_rica=Loc.CostaRica,
    croatia=Loc.Croatia,
    cuba=Loc.Cuba,
    curasao=Loc.CuraSao,
    cyprus=Loc.Cyprus,
    czech_republic=Loc.CzechRepublic,
    denmark=Loc.Denmark,
    diego_garcia=Loc.DiegoGarcia,
    djibouti=Loc.Djibouti,
    dominica=Loc.Dominica,
    dominican_republic=Loc.DominicanRepublic,
    east_timor=Loc.EastTimor,
    ecuador=Loc.Ecuador,
    egypt=Loc.Egypt,
    el_salvador=Loc.ElSalvador,
    equatorial_guinea=Loc.EquatorialGuinea,
    eritrea=Loc.Eritrea,
    estonia=Loc.Estonia,
    ethiopia=Loc.Ethiopia,
    european_union=Loc.EuropeanUnion,
    europe=Loc.Europe,
    falkland_islands=Loc.FalklandIslands,
    faroe_islands=Loc.FaroeIslands,
    fiji=Loc.Fiji,
    finland=Loc.Finland,
    france=Loc.France,
    french_guiana=Loc.FrenchGuiana,
    french_polynesia=Loc.FrenchPolynesia,
    french_southern_territories=Loc.FrenchSouthernTerritories,
    gabon=Loc.Gabon,
    gambia=Loc.Gambia,
    georgia=Loc.Georgia,
    germany=Loc.Germany,
    ghana=Loc.Ghana,
    gibraltar=Loc.Gibraltar,
    greece=Loc.Greece,
    greenland=Loc.Greenland,
    grenada=Loc.Grenada,
    guadeloupe=Loc.Guadeloupe,
    guam=Loc.Guam,
    guatemala=Loc.Guatemala,
    guernsey=Loc.Guernsey,
    guinea=Loc.Guinea,
    guineabissau=Loc.GuineaBissau,
    guyana=Loc.Guyana,
    haiti=Loc.Haiti,
    heard_and_mcdonald_islands=Loc.HeardAndMcDonaldIslands,
    honduras=Loc.Honduras,
    hongkong=Loc.HongKong,
    hungary=Loc.Hungary,
    iceland=Loc.Iceland,
    india=Loc.India,
    indonesia=Loc.Indonesia,
    iran=Loc.Iran,
    iraq=Loc.Iraq,
    ireland=Loc.Ireland,
    isle_of_man=Loc.IsleOfMan,
    israel=Loc.Israel,
    italy=Loc.Italy,
    ivory_coast=Loc.IvoryCoast,
    jamaica=Loc.Jamaica,
    japan=Loc.Japan,
    jersey=Loc.Jersey,
    jordan=Loc.Jordan,
    kazakhstan=Loc.Kazakhstan,
    kenya=Loc.Kenya,
    kiribati=Loc.Kiribati,
    kosovo=Loc.Kosovo,
    kuwait=Loc.Kuwait,
    kyrgyzstan=Loc.Kyrgyzstan,
    laos=Loc.Laos,
    latin_america=Loc.LatinAmerica,
    latvia=Loc.Latvia,
    lebanon=Loc.Lebanon,
    lesotho=Loc.Lesotho,
    liberia=Loc.Liberia,
    libya=Loc.Libya,
    liechtenstein=Loc.Liechtenstein,
    lithuania=Loc.Lithuania,
    luxembourg=Loc.Luxembourg,
    macau=Loc.Macau,
    macedonia=Loc.Macedonia,
    madagascar=Loc.Madagascar,
    malawi=Loc.Malawi,
    malaysia=Loc.Malaysia,
    maldives=Loc.Maldives,
    mali=Loc.Mali,
    malta=Loc.Malta,
    marshall_islands=Loc.MarshallIslands,
    martinique=Loc.Martinique,
    mauritania=Loc.Mauritania,
    mauritius=Loc.Mauritius,
    mayotte=Loc.Mayotte,
    mexico=Loc.Mexico,
    micronesia=Loc.Micronesia,
    moldova=Loc.Moldova,
    monaco=Loc.Monaco,
    mongolia=Loc.Mongolia,
    montenegro=Loc.Montenegro,
    montserrat=Loc.Montserrat,
    morocco=Loc.Morocco,
    mozambique=Loc.Mozambique,
    myanmar=Loc.Myanmar,
    namibia=Loc.Namibia,
    nauru_country=Loc.NauruCountry,
    nepal=Loc.Nepal,
    netherlands=Loc.Netherlands,
    new_caledonia=Loc.NewCaledonia,
    new_zealand=Loc.NewZealand,
    nicaragua=Loc.Nicaragua,
    niger=Loc.Niger,
    nigeria=Loc.Nigeria,
    niue=Loc.Niue,
    norfolk_island=Loc.NorfolkIsland,
    northern_mariana_islands=Loc.NorthernMarianaIslands,
    north_korea=Loc.NorthKorea,
    norway=Loc.Norway,
    oman=Loc.Oman,
    outlying_oceania=Loc.OutlyingOceania,
    pakistan=Loc.Pakistan,
    palau=Loc.Palau,
    palestinian_territories=Loc.PalestinianTerritories,
    panama=Loc.Panama,
    papua_new_guinea=Loc.PapuaNewGuinea,
    paraguay=Loc.Paraguay,
    peru=Loc.Peru,
    philippines=Loc.Philippines,
    pitcairn=Loc.Pitcairn,
    poland=Loc.Poland,
    portugal=Loc.Portugal,
    puertorico=Loc.PuertoRico,
    qatar=Loc.Qatar,
    reunion=Loc.Reunion,
    romania=Loc.Romania,
    russia=Loc.Russia,
    rwanda=Loc.Rwanda,
    saint_barthelemy=Loc.SaintBarthelemy,
    saint_helena=Loc.SaintHelena,
    saint_kitts_and_nevis=Loc.SaintKittsAndNevis,
    saint_lucia=Loc.SaintLucia,
    saint_martin=Loc.SaintMartin,
    saint_pierre_and_miquelon=Loc.SaintPierreAndMiquelon,
    saint_vincent_and_the_grenadines=Loc.SaintVincentAndTheGrenadines,
    samoa=Loc.Samoa,
    san_marino=Loc.SanMarino,
    sao_tome_and_principe=Loc.SaoTomeAndPrincipe,
    saudi_arabia=Loc.SaudiArabia,
    senegal=Loc.Senegal,
    serbia=Loc.Serbia,
    seychelles=Loc.Seychelles,
    sierra_leone=Loc.SierraLeone,
    singapore=Loc.Singapore,
    sint_maarten=Loc.SintMaarten,
    slovakia=Loc.Slovakia,
    slovenia=Loc.Slovenia,
    solomon_islands=Loc.SolomonIslands,
    somalia=Loc.Somalia,
    south_africa=Loc.SouthAfrica,
    south_georgia_and_south_sandwich_islands=Loc.SouthGeorgiaAndTheSouthSandwichIslands,
    south_korea=Loc.SouthKorea,
    south_sudan=Loc.SouthSudan,
    spain=Loc.Spain,
    sri_lanka=Loc.SriLanka,
    sudan=Loc.Sudan,
    suriname=Loc.Suriname,
    svalbard_and_jan_mayen_islands=Loc.SvalbardAndJanMayenIslands,
    swaziland=Loc.Swaziland,
    sweden=Loc.Sweden,
    switzerland=Loc.Switzerland,
    syria=Loc.Syria,
    taiwan=Loc.Taiwan,
    tajikistan=Loc.Tajikistan,
    tanzania=Loc.Tanzania,
    thailand=Loc.Thailand,
    togo=Loc.Togo,
    tokelau_country=Loc.TokelauCountry,
    tonga=Loc.Tonga,
    trinidad_and_tobago=Loc.TrinidadAndTobago,
    tristan_da_cunha=Loc.TristanDaCunha,
    tunisia=Loc.Tunisia,
    turkey=Loc.Turkey,
    turkmenistan=Loc.Turkmenistan,
    turks_and_caicos_islands=Loc.TurksAndCaicosIslands,
    tuvalu_country=Loc.TuvaluCountry,
    uganda=Loc.Uganda,
    ukraine=Loc.Ukraine,
    united_arabemirates=Loc.UnitedArabEmirates,
    united_kingdom=Loc.UnitedKingdom,
    united_states=Loc.UnitedStates,
    united_states_minor_outlying_islands=Loc.UnitedStatesMinorOutlyingIslands,
    united_states_virgin_islands=Loc.UnitedStatesVirginIslands,
    uruguay=Loc.Uruguay,
    uzbekistan=Loc.Uzbekistan,
    vanuatu=Loc.Vanuatu,
    vatican_city_state=Loc.VaticanCityState,
    venezuela=Loc.Venezuela,
    vietnam=Loc.Vietnam,
    wallis_and_futuna_islands=Loc.WallisAndFutunaIslands,
    western_sahara=Loc.WesternSahara,
    world=Loc.World,
    yemen=Loc.Yemen,
    zambia=Loc.Zambia,
    zimbabwe=Loc.Zimbabwe,
)


LANGUAGE = bidict(
    any_language=QtCore.QLocale.AnyLanguage,
    c=QtCore.QLocale.C,
    abkhazian=QtCore.QLocale.Abkhazian,
    afar=QtCore.QLocale.Afar,
    afrikaans=QtCore.QLocale.Afrikaans,
    aghem=QtCore.QLocale.Aghem,
    akan=QtCore.QLocale.Akan,
    akkadian=QtCore.QLocale.Akkadian,
    akoose=QtCore.QLocale.Akoose,
    albanian=QtCore.QLocale.Albanian,
    american_sign_language=QtCore.QLocale.AmericanSignLanguage,
    amharic=QtCore.QLocale.Amharic,
    ancient_egyptian=QtCore.QLocale.AncientEgyptian,
    ancient_greek=QtCore.QLocale.AncientGreek,
    arabic=QtCore.QLocale.Arabic,
    aragonese=QtCore.QLocale.Aragonese,
    aramaic=QtCore.QLocale.Aramaic,
    armenian=QtCore.QLocale.Armenian,
    assamese=QtCore.QLocale.Assamese,
    asturian=QtCore.QLocale.Asturian,
    asu=QtCore.QLocale.Asu,
    atsam=QtCore.QLocale.Atsam,
    avaric=QtCore.QLocale.Avaric,
    avestan=QtCore.QLocale.Avestan,
    aymara=QtCore.QLocale.Aymara,
    azerbaijani=QtCore.QLocale.Azerbaijani,
    bafia=QtCore.QLocale.Bafia,
    balinese=QtCore.QLocale.Balinese,
    bambara=QtCore.QLocale.Bambara,
    bamun=QtCore.QLocale.Bamun,
    basaa=QtCore.QLocale.Basaa,
    bashkir=QtCore.QLocale.Bashkir,
    basque=QtCore.QLocale.Basque,
    bataktoba=QtCore.QLocale.BatakToba,
    belarusian=QtCore.QLocale.Belarusian,
    bemba=QtCore.QLocale.Bemba,
    bena=QtCore.QLocale.Bena,
    bengali=QtCore.QLocale.Bengali,
    bhojpuri=QtCore.QLocale.Bhojpuri,
    bislama=QtCore.QLocale.Bislama,
    blin=QtCore.QLocale.Blin,
    bodo=QtCore.QLocale.Bodo,
    bosnian=QtCore.QLocale.Bosnian,
    breton=QtCore.QLocale.Breton,
    buginese=QtCore.QLocale.Buginese,
    bulgarian=QtCore.QLocale.Bulgarian,
    burmese=QtCore.QLocale.Burmese,
    cantonese=QtCore.QLocale.Cantonese,
    catalan=QtCore.QLocale.Catalan,
    cebuano=QtCore.QLocale.Cebuano,
    central_kurdish=QtCore.QLocale.CentralKurdish,
    central_morocco_tamazight=QtCore.QLocale.CentralMoroccoTamazight,
    chakma=QtCore.QLocale.Chakma,
    chamorro=QtCore.QLocale.Chamorro,
    chechen=QtCore.QLocale.Chechen,
    cherokee=QtCore.QLocale.Cherokee,
    chickasaw=QtCore.QLocale.Chickasaw,
    chiga=QtCore.QLocale.Chiga,
    chinese=QtCore.QLocale.Chinese,
    church=QtCore.QLocale.Church,
    chuvash=QtCore.QLocale.Chuvash,
    colognian=QtCore.QLocale.Colognian,
    coptic=QtCore.QLocale.Coptic,
    cornish=QtCore.QLocale.Cornish,
    corsican=QtCore.QLocale.Corsican,
    cree=QtCore.QLocale.Cree,
    croatian=QtCore.QLocale.Croatian,
    czech=QtCore.QLocale.Czech,
    danish=QtCore.QLocale.Danish,
    divehi=QtCore.QLocale.Divehi,
    dogri=QtCore.QLocale.Dogri,
    duala=QtCore.QLocale.Duala,
    dutch=QtCore.QLocale.Dutch,
    dzongkha=QtCore.QLocale.Dzongkha,
    embu=QtCore.QLocale.Embu,
    english=QtCore.QLocale.English,
    erzya=QtCore.QLocale.Erzya,
    esperanto=QtCore.QLocale.Esperanto,
    estonian=QtCore.QLocale.Estonian,
    ewe=QtCore.QLocale.Ewe,
    ewondo=QtCore.QLocale.Ewondo,
    faroese=QtCore.QLocale.Faroese,
    fijian=QtCore.QLocale.Fijian,
    filipino=QtCore.QLocale.Filipino,
    finnish=QtCore.QLocale.Finnish,
    french=QtCore.QLocale.French,
    friulian=QtCore.QLocale.Friulian,
    fulah=QtCore.QLocale.Fulah,
    ga=QtCore.QLocale.Ga,
    gaelic=QtCore.QLocale.Gaelic,
    galician=QtCore.QLocale.Galician,
    ganda=QtCore.QLocale.Ganda,
    geez=QtCore.QLocale.Geez,
    georgian=QtCore.QLocale.Georgian,
    german=QtCore.QLocale.German,
    gothic=QtCore.QLocale.Gothic,
    greek=QtCore.QLocale.Greek,
    greenlandic=QtCore.QLocale.Greenlandic,
    guarani=QtCore.QLocale.Guarani,
    gujarati=QtCore.QLocale.Gujarati,
    gusii=QtCore.QLocale.Gusii,
    haitian=QtCore.QLocale.Haitian,
    hausa=QtCore.QLocale.Hausa,
    hawaiian=QtCore.QLocale.Hawaiian,
    hebrew=QtCore.QLocale.Hebrew,
    herero=QtCore.QLocale.Herero,
    hindi=QtCore.QLocale.Hindi,
    hirimotu=QtCore.QLocale.HiriMotu,
    hungarian=QtCore.QLocale.Hungarian,
    icelandic=QtCore.QLocale.Icelandic,
    ido=QtCore.QLocale.Ido,
    igbo=QtCore.QLocale.Igbo,
    inari_sami=QtCore.QLocale.InariSami,
    indonesian=QtCore.QLocale.Indonesian,
    ingush=QtCore.QLocale.Ingush,
    interlingua=QtCore.QLocale.Interlingua,
    interlingue=QtCore.QLocale.Interlingue,
    inuktitut=QtCore.QLocale.Inuktitut,
    inupiak=QtCore.QLocale.Inupiak,
    irish=QtCore.QLocale.Irish,
    italian=QtCore.QLocale.Italian,
    japanese=QtCore.QLocale.Japanese,
    javanese=QtCore.QLocale.Javanese,
    jju=QtCore.QLocale.Jju,
    jola_fonyi=QtCore.QLocale.JolaFonyi,
    kabuverdianu=QtCore.QLocale.Kabuverdianu,
    kabyle=QtCore.QLocale.Kabyle,
    kako=QtCore.QLocale.Kako,
    kalenjin=QtCore.QLocale.Kalenjin,
    kamba=QtCore.QLocale.Kamba,
    kannada=QtCore.QLocale.Kannada,
    kanuri=QtCore.QLocale.Kanuri,
    kashmiri=QtCore.QLocale.Kashmiri,
    kazakh=QtCore.QLocale.Kazakh,
    kenyang=QtCore.QLocale.Kenyang,
    khmer=QtCore.QLocale.Khmer,
    kiche=QtCore.QLocale.Kiche,
    kikuyu=QtCore.QLocale.Kikuyu,
    kinyarwanda=QtCore.QLocale.Kinyarwanda,
    kirghiz=QtCore.QLocale.Kirghiz,
    komi=QtCore.QLocale.Komi,
    kongo=QtCore.QLocale.Kongo,
    konkani=QtCore.QLocale.Konkani,
    korean=QtCore.QLocale.Korean,
    koro=QtCore.QLocale.Koro,
    koyraboro_senni=QtCore.QLocale.KoyraboroSenni,
    koyra_chiini=QtCore.QLocale.KoyraChiini,
    kpelle=QtCore.QLocale.Kpelle,
    kurdish=QtCore.QLocale.Kurdish,
    kwanyama=QtCore.QLocale.Kwanyama,
    kwasio=QtCore.QLocale.Kwasio,
    lakota=QtCore.QLocale.Lakota,
    langi=QtCore.QLocale.Langi,
    lao=QtCore.QLocale.Lao,
    latin=QtCore.QLocale.Latin,
    latvian=QtCore.QLocale.Latvian,
    lezghian=QtCore.QLocale.Lezghian,
    limburgish=QtCore.QLocale.Limburgish,
    lingala=QtCore.QLocale.Lingala,
    literary_chinese=QtCore.QLocale.LiteraryChinese,
    lithuanian=QtCore.QLocale.Lithuanian,
    lojban=QtCore.QLocale.Lojban,
    lower_sorbian=QtCore.QLocale.LowerSorbian,
    low_german=QtCore.QLocale.LowGerman,
    luba_katanga=QtCore.QLocale.LubaKatanga,
    lule_sami=QtCore.QLocale.LuleSami,
    luo=QtCore.QLocale.Luo,
    luxembourgish=QtCore.QLocale.Luxembourgish,
    luyia=QtCore.QLocale.Luyia,
    macedonian=QtCore.QLocale.Macedonian,
    machame=QtCore.QLocale.Machame,
    maithili=QtCore.QLocale.Maithili,
    makhuwa_meetto=QtCore.QLocale.MakhuwaMeetto,
    makonde=QtCore.QLocale.Makonde,
    malagasy=QtCore.QLocale.Malagasy,
    malay=QtCore.QLocale.Malay,
    malayalam=QtCore.QLocale.Malayalam,
    maltese=QtCore.QLocale.Maltese,
    mandingo=QtCore.QLocale.Mandingo,
    manipuri=QtCore.QLocale.Manipuri,
    manx=QtCore.QLocale.Manx,
    maori=QtCore.QLocale.Maori,
    mapuche=QtCore.QLocale.Mapuche,
    marathi=QtCore.QLocale.Marathi,
    marshallese=QtCore.QLocale.Marshallese,
    masai=QtCore.QLocale.Masai,
    mazanderani=QtCore.QLocale.Mazanderani,
    mende=QtCore.QLocale.Mende,
    meru=QtCore.QLocale.Meru,
    meta=QtCore.QLocale.Meta,
    mohawk=QtCore.QLocale.Mohawk,
    mongolian=QtCore.QLocale.Mongolian,
    morisyen=QtCore.QLocale.Morisyen,
    mundang=QtCore.QLocale.Mundang,
    muscogee=QtCore.QLocale.Muscogee,
    nama=QtCore.QLocale.Nama,
    nauru_language=QtCore.QLocale.NauruLanguage,
    navaho=QtCore.QLocale.Navaho,
    ndonga=QtCore.QLocale.Ndonga,
    nepali=QtCore.QLocale.Nepali,
    newari=QtCore.QLocale.Newari,
    ngiemboon=QtCore.QLocale.Ngiemboon,
    ngomba=QtCore.QLocale.Ngomba,
    nko=QtCore.QLocale.Nko,
    northern_luri=QtCore.QLocale.NorthernLuri,
    northern_sami=QtCore.QLocale.NorthernSami,
    northern_sotho=QtCore.QLocale.NorthernSotho,
    north_ndebele=QtCore.QLocale.NorthNdebele,
    norwegian_bokmal=QtCore.QLocale.NorwegianBokmal,
    norwegian_nynorsk=QtCore.QLocale.NorwegianNynorsk,
    nuer=QtCore.QLocale.Nuer,
    nyanja=QtCore.QLocale.Nyanja,
    nyankole=QtCore.QLocale.Nyankole,
    occitan=QtCore.QLocale.Occitan,
    ojibwa=QtCore.QLocale.Ojibwa,
    old_irish=QtCore.QLocale.OldIrish,
    old_norse=QtCore.QLocale.OldNorse,
    old_persian=QtCore.QLocale.OldPersian,
    oriya=QtCore.QLocale.Oriya,
    oromo=QtCore.QLocale.Oromo,
    osage=QtCore.QLocale.Osage,
    ossetic=QtCore.QLocale.Ossetic,
    pahlavi=QtCore.QLocale.Pahlavi,
    palauan=QtCore.QLocale.Palauan,
    pali=QtCore.QLocale.Pali,
    papiamento=QtCore.QLocale.Papiamento,
    pashto=QtCore.QLocale.Pashto,
    persian=QtCore.QLocale.Persian,
    phoenician=QtCore.QLocale.Phoenician,
    polish=QtCore.QLocale.Polish,
    portuguese=QtCore.QLocale.Portuguese,
    prussian=QtCore.QLocale.Prussian,
    punjabi=QtCore.QLocale.Punjabi,
    quechua=QtCore.QLocale.Quechua,
    romanian=QtCore.QLocale.Romanian,
    romansh=QtCore.QLocale.Romansh,
    rombo=QtCore.QLocale.Rombo,
    rundi=QtCore.QLocale.Rundi,
    russian=QtCore.QLocale.Russian,
    rwa=QtCore.QLocale.Rwa,
    saho=QtCore.QLocale.Saho,
    sakha=QtCore.QLocale.Sakha,
    samburu=QtCore.QLocale.Samburu,
    samoan=QtCore.QLocale.Samoan,
    sango=QtCore.QLocale.Sango,
    sangu=QtCore.QLocale.Sangu,
    sanskrit=QtCore.QLocale.Sanskrit,
    santali=QtCore.QLocale.Santali,
    sardinian=QtCore.QLocale.Sardinian,
    saurashtra=QtCore.QLocale.Saurashtra,
    sena=QtCore.QLocale.Sena,
    serbian=QtCore.QLocale.Serbian,
    shambala=QtCore.QLocale.Shambala,
    shona=QtCore.QLocale.Shona,
    sichuanyi=QtCore.QLocale.SichuanYi,
    sicilian=QtCore.QLocale.Sicilian,
    sidamo=QtCore.QLocale.Sidamo,
    silesian=QtCore.QLocale.Silesian,
    sindhi=QtCore.QLocale.Sindhi,
    sinhala=QtCore.QLocale.Sinhala,
    skoltsami=QtCore.QLocale.SkoltSami,
    slovak=QtCore.QLocale.Slovak,
    slovenian=QtCore.QLocale.Slovenian,
    soga=QtCore.QLocale.Soga,
    somali=QtCore.QLocale.Somali,
    southern_kurdish=QtCore.QLocale.SouthernKurdish,
    southern_sami=QtCore.QLocale.SouthernSami,
    southern_sotho=QtCore.QLocale.SouthernSotho,
    south_ndebele=QtCore.QLocale.SouthNdebele,
    spanish=QtCore.QLocale.Spanish,
    standard_moroccan_tamazight=QtCore.QLocale.StandardMoroccanTamazight,
    sundanese=QtCore.QLocale.Sundanese,
    swahili=QtCore.QLocale.Swahili,
    swati=QtCore.QLocale.Swati,
    swedish=QtCore.QLocale.Swedish,
    swiss_german=QtCore.QLocale.SwissGerman,
    syriac=QtCore.QLocale.Syriac,
    tachelhit=QtCore.QLocale.Tachelhit,
    tahitian=QtCore.QLocale.Tahitian,
    taidam=QtCore.QLocale.TaiDam,
    taita=QtCore.QLocale.Taita,
    tajik=QtCore.QLocale.Tajik,
    tamil=QtCore.QLocale.Tamil,
    taroko=QtCore.QLocale.Taroko,
    tasawaq=QtCore.QLocale.Tasawaq,
    tatar=QtCore.QLocale.Tatar,
    telugu=QtCore.QLocale.Telugu,
    teso=QtCore.QLocale.Teso,
    thai=QtCore.QLocale.Thai,
    tibetan=QtCore.QLocale.Tibetan,
    tigre=QtCore.QLocale.Tigre,
    tigrinya=QtCore.QLocale.Tigrinya,
    tokelau_language=QtCore.QLocale.TokelauLanguage,
    tokpisin=QtCore.QLocale.TokPisin,
    tongan=QtCore.QLocale.Tongan,
    tsonga=QtCore.QLocale.Tsonga,
    tswana=QtCore.QLocale.Tswana,
    turkish=QtCore.QLocale.Turkish,
    turkmen=QtCore.QLocale.Turkmen,
    tuvalu_language=QtCore.QLocale.TuvaluLanguage,
    tyap=QtCore.QLocale.Tyap,
    ugaritic=QtCore.QLocale.Ugaritic,
    uighur=QtCore.QLocale.Uighur,
    ukrainian=QtCore.QLocale.Ukrainian,
    upper_sorbian=QtCore.QLocale.UpperSorbian,
    urdu=QtCore.QLocale.Urdu,
    uzbek=QtCore.QLocale.Uzbek,
    vai=QtCore.QLocale.Vai,
    venda=QtCore.QLocale.Venda,
    vietnamese=QtCore.QLocale.Vietnamese,
    volapuk=QtCore.QLocale.Volapuk,
    vunjo=QtCore.QLocale.Vunjo,
    walamo=QtCore.QLocale.Walamo,
    walloon=QtCore.QLocale.Walloon,
    walser=QtCore.QLocale.Walser,
    warlpiri=QtCore.QLocale.Warlpiri,
    welsh=QtCore.QLocale.Welsh,
    western_balochi=QtCore.QLocale.WesternBalochi,
    western_frisian=QtCore.QLocale.WesternFrisian,
    wolof=QtCore.QLocale.Wolof,
    xhosa=QtCore.QLocale.Xhosa,
    yangben=QtCore.QLocale.Yangben,
    yiddish=QtCore.QLocale.Yiddish,
    yoruba=QtCore.QLocale.Yoruba,
    zarma=QtCore.QLocale.Zarma,
    zhuang=QtCore.QLocale.Zhuang,
    zulu=QtCore.QLocale.Zulu,
)


class Locale(QtCore.QLocale):
    def __repr__(self):
        return f"{type(self).__name__}({self.bcp47Name()!r})"

    def __reduce__(self):
        return self.__class__, (self.bcp47Name(),)


if __name__ == "__main__":
    locale = Locale()
    print(repr(locale))



"""Module containing helper functions."""

import sys
from typing import Iterable, TYPE_CHECKING
import traceback
import logging

import bidict as bdct

if TYPE_CHECKING:
    from qtpy import QtCore
    import datetime

logger = logging.getLogger(__name__)


def to_datetime(date: "QtCore.QDateTime") -> "datetime.datetime":
    try:
        return date.toPython()  # pyqt5
    except (AttributeError, TypeError):
        return date.toPyDateTime()


def to_date(date: "QtCore.QDate") -> "datetime.date":
    try:
        return date.toPython()  # pyqt5
    except (AttributeError, TypeError):
        return date.toPyDate()


def to_time(time: "QtCore.QTime") -> "datetime.time":
    try:
        return time.toPython()  # pyqt5
    except (AttributeError, TypeError):
        return time.toPyTime()


class bidict(bdct.bidict):
    def __init__(self, *args, **kwargs):
        if len(args) == 1 and isinstance(args[0], dict):
            super().__init__(args[0])
        else:
            super().__init__(kwargs)


class InvalidParamError(ValueError):
    """Exception raised for invalid params in method calls.

    Args:
        value: param value which caused the error
        valid_options: allowed options
    """

    def __init__(self, value, valid_options: Iterable):
        self.value = value
        opts = " / ".join(repr(opt) for opt in valid_options)
        self.message = f"Invalid value: {value!r}. Allowed options are {opts}."
        super().__init__(self.message)


def install_exceptionhook(debug=False):
    def handleException(exc_type, exc_value, exc_traceback):
        """Causes the application to quit in case of an unhandled exception.

        Shows an error dialog before quitting when not in debugging mode.
        """
        logger.critical(
            f"Bug: uncaught {exc_type.__name__}",
            exc_info=(exc_type, exc_value, exc_traceback),
        )
        if debug:
            sys.exit(1)
        else:
            from prettyqt import widgets

            # Constructing a QApplication in case this hasn't been done yet.
            _ = widgets.app()
            lst = traceback.format_exception(exc_type, exc_value, exc_traceback)
            msgBox = widgets.MessageBox(
                icon="warning",
                text=f"Bug: uncaught {exc_type.__name__}",
                informative_text=str(exc_value),
                details="".join(lst),
            )
            msgBox.exec_()
            sys.exit(1)

    sys.excepthook = handleException



from typing import List
from qtpy import QtMultimedia

from prettyqt import core, multimedia
from prettyqt.utils import bidict, InvalidParamError


FOCUS_MODES = bidict(
    manual=QtMultimedia.QCameraFocus.ManualFocus,
    hyperfocal=QtMultimedia.QCameraFocus.HyperfocalFocus,
    infinity=QtMultimedia.QCameraFocus.InfinityFocus,
    auto=QtMultimedia.QCameraFocus.AutoFocus,
    continuous=QtMultimedia.QCameraFocus.ContinuousFocus,
    macro=QtMultimedia.QCameraFocus.MacroFocus,
)

FOCUS_POINT_MODES = bidict(
    auto=QtMultimedia.QCameraFocus.FocusPointAuto,
    center=QtMultimedia.QCameraFocus.FocusPointCenter,
    face_detection=QtMultimedia.QCameraFocus.FocusPointFaceDetection,
    custom=QtMultimedia.QCameraFocus.FocusPointCustom,
)

QtMultimedia.QCameraFocus.__bases__ = (core.Object,)


class CameraFocus(core.Object):
    def __init__(self, item: QtMultimedia.QCameraFocus):
        self.item = item

    def __getattr__(self, val):
        return getattr(self.item, val)

    def set_focus_mode(self, mode: str):
        """Set the focus mode.

        Allowed values are "manual", "hyperfocal", "infinity", "auto", "continuous",
                           "macro"

        Args:
            mode: focus mode

        Raises:
            InvalidParamError: focus mode does not exist
        """
        if mode not in FOCUS_MODES:
            raise InvalidParamError(mode, FOCUS_MODES)
        self.item.setFocusMode(FOCUS_MODES[mode])

    def get_focus_mode(self) -> str:
        """Return current focus mode.

        Possible values: "manual", "hyperfocal", "infinity", "auto", "continuous",
                         "macro"

        Returns:
            focus mode
        """
        return FOCUS_MODES.inverse[self.item.focusMode()]

    def set_focus_point_mode(self, mode: str):
        """Set the focus mode.

        Allowed values are "auto", "center", "face_detection", "custom"

        Args:
            mode: focus point mode

        Raises:
            InvalidParamError: focus point mode does not exist
        """
        if mode not in FOCUS_POINT_MODES:
            raise InvalidParamError(mode, FOCUS_POINT_MODES)
        self.item.setFocusPointMode(FOCUS_POINT_MODES[mode])

    def get_focus_point_mode(self) -> str:
        """Return current focus point mode.

        Possible values: "auto", "center", "face_detection", "custom"

        Returns:
            focus point mode
        """
        return FOCUS_POINT_MODES.inverse[self.item.focusPointMode()]

    def get_custom_focus_point(self) -> core.PointF:
        return core.PointF(self.item.customFocusPoint())

    def is_focus_mode_supported(self, mode: str) -> bool:
        if mode not in FOCUS_MODES:
            raise InvalidParamError(mode, FOCUS_MODES)
        return self.item.isFocusModeSupported(FOCUS_MODES[mode])

    def is_focus_point_mode_supported(self, mode: str) -> bool:
        if mode not in FOCUS_POINT_MODES:
            raise InvalidParamError(mode, FOCUS_POINT_MODES)
        return self.item.isFocusPointModeSupported(FOCUS_POINT_MODES[mode])

    def get_focus_zones(self) -> List[multimedia.CameraFocusZone]:
        return [multimedia.CameraFocusZone(i) for i in self.item.focusZones()]


if __name__ == "__main__":
    cam = multimedia.Camera()
    focus = cam.get_focus()



from __future__ import annotations

from typing import Literal, Callable

from qtpy import QtCore

from prettyqt import core
from prettyqt.utils import bidict, InvalidParamError


QtCore.QTimer.__bases__ = (core.Object,)

TYPE = bidict(
    precise=QtCore.Qt.PreciseTimer,
    coarse=QtCore.Qt.CoarseTimer,
    very_coarse=QtCore.Qt.VeryCoarseTimer,
)

TypeStr = Literal["precise", "coarse", "very_coarse"]


class Timer(QtCore.QTimer):
    @classmethod
    def single_shot(cls, callback: Callable) -> Timer:
        timer = cls()
        timer.timeout.connect(callback)
        timer.setSingleShot(True)
        return timer

    def set_type(self, typ: TypeStr):
        """Set the timer type.

        Args:
            typ: timer type

        Raises:
            InvalidParamError: timer type does not exist
        """
        if typ not in TYPE:
            raise InvalidParamError(typ, TYPE)
        self.setTimerType(TYPE[typ])

    def get_type(self) -> TypeStr:
        """Return current timer type.

        Returns:
            timer type
        """
        return TYPE.inverse[self.timerType()]



from __future__ import annotations

from dataclasses import dataclass, asdict
from typing import Tuple, Dict, Any, Iterator

from qtpy import QtMultimedia

from prettyqt import multimedia
from prettyqt.utils import InvalidParamError


@dataclass
class Settings:
    maximum_framerate: int
    minimum_framerate: int
    pixel_aspect_ratio: Tuple[int, int]
    pixel_format: str
    resolution: Tuple[int, int]


class CameraViewfinderSettings(QtMultimedia.QCameraViewfinderSettings):
    def __getitem__(self, index: str):
        return self.to_dict()[index]

    def __setitem__(self, index: str, value):
        if index == "maximum_framerate":
            self.setMaximumFrameRate(value)
        elif index == "minimum_framerate":
            self.setMinimumFrameRate(value)
        elif index == "pixel_aspect_ratio":
            self.setPixelAspectRatio(*value)
        elif index == "pixel_format":
            self.set_pixel_format(value)
        elif index == "resolution":
            self.setResolution(*value)

    def __iter__(self) -> Iterator[str]:
        return iter(self.to_dict().keys())

    def __len__(self):
        return len(self.to_dict())

    def set_pixel_format(self, fmt: str):
        if fmt not in multimedia.videoframe.PIXEL_FORMAT:
            raise InvalidParamError(fmt, multimedia.videoframe.PIXEL_FORMAT)
        self.setPixelFormat(multimedia.videoframe.PIXEL_FORMAT[fmt])

    def get_pixel_format(self) -> str:
        return multimedia.videoframe.PIXEL_FORMAT.inverse[self.pixelFormat()]

    def to_dataclass(self) -> Settings:
        size = self.resolution()
        ar = self.pixelAspectRatio()
        return Settings(
            maximum_framerate=self.maximumFrameRate(),
            minimum_framerate=self.minimumFrameRate(),
            pixel_aspect_ratio=(ar.width(), ar.height()),
            pixel_format=self.get_pixel_format(),
            resolution=(size.width(), size.height()),
        )

    def to_dict(self) -> Dict[str, Any]:
        return asdict(self.to_dataclass())

    @classmethod
    def from_dataclass(cls, data: Settings) -> CameraViewfinderSettings:
        instance = cls()
        instance.setMaximumFrameRate(data.maximum_framerate)
        instance.setMinimumFrameRate(data.minimum_framerate)
        instance.setPixelAspectRatio(*data.pixel_aspect_ratio)
        instance.set_pixel_format(data.pixel_format)
        instance.setResolution(*data.resolution)
        return instance

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> CameraViewfinderSettings:
        settings = Settings(**data)
        return cls.from_dataclass(settings)



from qtpy import QtNetwork

from prettyqt import core, network
from prettyqt.utils import bidict, InvalidParamError, helpers


SOCKET_OPTION = bidict(
    none=QtNetwork.QLocalServer.SocketOption(0),  # QtNetwork.QLocalServer.NoOptions
    user=QtNetwork.QLocalServer.UserAccessOption,
    group=QtNetwork.QLocalServer.GroupAccessOption,
    other=QtNetwork.QLocalServer.OtherAccessOption,
    world=QtNetwork.QLocalServer.WorldAccessOption,
)

QtNetwork.QLocalServer.__bases__ = (core.Object,)


class LocalServer(QtNetwork.QLocalServer):
    def get_server_error(self) -> str:
        return network.abstractsocket.SOCKET_ERROR.inverse[self.serverError()]

    def set_socket_options(self, *name: str):
        for item in name:
            if item not in SOCKET_OPTION:
                raise InvalidParamError(item, SOCKET_OPTION)
        flags = helpers.merge_flags(name, SOCKET_OPTION)
        self.setSocketOptions(flags)

    def get_socket_options(self):
        print(self.socketOptions())
        return [k for k, v in SOCKET_OPTION.items() if v & self.socketOptions()]


if __name__ == "__main__":
    server = LocalServer()



from __future__ import annotations

from typing import List, Iterator

from qtpy import QtLocation

from prettyqt import location


QtLocation.QPlaceMatchReply.__bases__ = (location.PlaceReply,)


class PlaceMatchReply(QtLocation.QPlaceMatchReply):
    def __iter__(self) -> Iterator[location.Place]:
        return iter(self.get_places())

    def __getitem__(self, index: int) -> location.Place:
        return self.get_places()[index]

    def __len__(self):
        return len(self.get_places())

    @classmethod
    def clone_from(cls, obj: QtLocation.QPlaceMatchReply) -> PlaceMatchReply:
        reply = cls(obj.parent())
        reply.setPlaces([location.Place(i) for i in obj.places()])
        request = location.PlaceMatchRequest(obj.request())
        reply.setRequest(request)
        return reply

    def get_places(self) -> List[location.Place]:
        return [location.Place(i) for i in self.places()]

    def get_request(self) -> location.PlaceMatchRequest:
        return location.PlaceMatchRequest(self.request())


if __name__ == "__main__":
    reply = PlaceMatchReply()



from qtpy import QtGui

from prettyqt import core
from prettyqt.utils import bidict, InvalidParamError

RESTART_HINT = bidict(
    if_running=QtGui.QSessionManager.RestartIfRunning,
    anyway=QtGui.QSessionManager.RestartAnyway,
    immediately=QtGui.QSessionManager.RestartImmediately,
    never=QtGui.QSessionManager.RestartNever,
)


QtGui.QSessionManager.__bases__ = (core.Object,)


class SessionManager:
    def __init__(self, item: QtGui.QSessionManager):
        self.item = item

    def __getattr__(self, val):
        return getattr(self.item, val)

    def set_restart_hint(self, style: str):
        """Set the restart hint.

        Allowed values are "if_running", "anyway", "immediatly", "never"

        Args:
            style: restart hint

        Raises:
            InvalidParamError: restart hint does not exist
        """
        if style not in RESTART_HINT:
            raise InvalidParamError(style, RESTART_HINT)
        self.setRestartHint(RESTART_HINT[style])

    def get_restart_hint(self) -> str:
        """Return current restart hint.

        Possible values: "if_running", "anyway", "immediatly", "never"

        Returns:
            restart hint
        """
        return RESTART_HINT.inverse[self.restartHint()]



from typing import Iterator
from qtpy import QtCore


class MarginsF(QtCore.QMarginsF):
    def __repr__(self):
        return (
            f"{type(self).__name__}({self.left()}, {self.top()}, {self.right()}, "
            f"{self.bottom()})"
        )

    def __reduce__(self):
        return self.__class__, (self.left(), self.top(), self.right(), self.bottom())

    def __bool__(self):
        return not self.isNull()

    def __iter__(self) -> Iterator[float]:
        yield self.left()
        yield self.top()
        yield self.right()
        yield self.bottom()


if __name__ == "__main__":
    margins = MarginsF(0, 0, 0, 0)
    print(repr(margins))



"""syntaxhighlighters module.

contains some custom syntax highlighers
"""

from .highlightrule import HighlightRule
from .jsonhighlighter import JsonHighlighter
from .pythonhighlighter import PythonHighlighter
from .yamlhighlighter import YamlHighlighter
from .xmlhighlighter import XmlHighlighter
from .regexmatchhighlighter import RegexMatchHighlighter
from .markdownhighlighter import MarkdownHighlighter
from .pygmentshighlighter import PygmentsHighlighter


__all__ = [
    "HighlightRule",
    "JsonHighlighter",
    "YamlHighlighter",
    "PythonHighlighter",
    "XmlHighlighter",
    "RegexMatchHighlighter",
    "MarkdownHighlighter",
    "PygmentsHighlighter",
]



from typing import Union, Callable, Literal

from qtpy import QtCore

from prettyqt import core
from prettyqt.utils import bidict, InvalidParamError

DIRECTION = bidict(forward=QtCore.QTimeLine.Forward, backward=QtCore.QTimeLine.Backward)

DirectionStr = Literal["forward", "backward"]

STATE = bidict(
    not_running=QtCore.QTimeLine.NotRunning,
    paused=QtCore.QTimeLine.Paused,
    running=QtCore.QTimeLine.Running,
)

StateStr = Literal["not_running", "paused", "running"]

QtCore.QTimeLine.__bases__ = (core.Object,)


class TimeLine(QtCore.QTimeLine):
    def serialize_fields(self):
        return dict(
            current_time=self.currentTime(),
            direction=self.get_direction(),
            duration=self.duration(),
            easing_curve=self.easingCurve(),
            loop_count=self.loopCount(),
            update_interval=self.updateInterval(),
        )

    def set_direction(self, direction: DirectionStr):
        """Set the direction.

        Args:
            direction: direction

        Raises:
            InvalidParamError: direction does not exist
        """
        if direction not in DIRECTION:
            raise InvalidParamError(direction, DIRECTION)
        self.setDirection(DIRECTION[direction])

    def get_direction(self) -> DirectionStr:
        """Return current direction.

        Returns:
            direction
        """
        return DIRECTION.inverse[self.direction()]

    def get_state(self) -> StateStr:
        """Return current state.

        Returns:
            state
        """
        return STATE.inverse[self.state()]

    def set_easing(
        self, easing_type: Union[core.easingcurve.TypeStr, Callable[[float], float]]
    ) -> core.EasingCurve:
        curve = core.EasingCurve()
        if isinstance(easing_type, str):
            curve.set_type(easing_type)
        else:
            curve.set_custom_type(easing_type)
        self.setEasingCurve(curve)
        return curve

    def get_easing(self) -> Union[str, Callable[[float], float]]:
        curve = core.EasingCurve(self.easingCurve())
        typ = curve.get_type()
        if typ == "custom":
            return curve.get_custom_type()
        else:
            return typ



from __future__ import annotations

import sys

from qtpy import QtCore
import qtpy


class VersionNumber(QtCore.QVersionNumber):
    def __init__(self, *args, **kwargs):
        if len(args) == 1:
            if isinstance(args[0], str):
                args = tuple(int(i) for i in args[0].split("."))
            elif isinstance(args[0], tuple):
                args = args[0]
        super().__init__(*args, **kwargs)

    def __repr__(self):
        return f"{type(self).__name__}({self.major()}, {self.minor()}, {self.micro()})"

    def __reduce__(self):
        return self.__class__, (self.major(), self.minor(), self.micro())

    def __str__(self):
        return self.toString()

    def __eq__(self, other):
        if isinstance(other, (str, tuple)):
            other = VersionNumber(other)
        return super().__eq__(other)

    def __gt__(self, other):
        if isinstance(other, (str, tuple)):
            other = VersionNumber(other)
        return super().__gt__(other)

    def __ge__(self, other):
        if isinstance(other, (str, tuple)):
            other = VersionNumber(other)
        return super().__ge__(other)

    def __lt__(self, other):
        if isinstance(other, (str, tuple)):
            other = VersionNumber(other)
        return super().__lt__(other)

    def __le__(self, other):
        if isinstance(other, (str, tuple)):
            other = VersionNumber(other)
        return super().__le__(other)

    @classmethod
    def get_qt_version(cls) -> VersionNumber:
        return cls(*[int(i) for i in qtpy.QT_VERSION.split(".")])

    @classmethod
    def get_python_version(cls) -> VersionNumber:
        return cls(*sys.version_info[:3])

    def major(self) -> int:
        """An integer representing the major version."""
        return self.majorVersion()

    def minor(self) -> int:
        """An integer representing the minor version."""
        return self.minorVersion()

    def micro(self) -> int:
        """An integer representing the micro version."""
        return self.microVersion()



from __future__ import annotations

from qtpy import QtCore


class SizeF(QtCore.QSizeF):
    def __repr__(self):
        return f"{type(self).__name__}({self.width()}, {self.height()})"

    def __getitem__(self, index) -> float:
        return (self.width(), self.height())[index]

    def __reduce__(self):
        return self.__class__, (self.width(), self.height())

    def expanded_to(self, size: QtCore.QSizeF) -> SizeF:
        return SizeF(self.expandedTo(size))


if __name__ == "__main__":
    size = SizeF(10, 20)
    print(tuple(size))
    size = size.expanded_to(QtCore.QSizeF(100, 100))
    print(type(size))



from typing import Literal, List, Union

import datetime

from qtpy import QtWidgets, QtCore

from prettyqt import core, widgets
from prettyqt.utils import to_datetime, to_date, to_time, bidict, InvalidParamError


SECTIONS = bidict(
    none=QtWidgets.QDateTimeEdit.NoSection,
    am_pm=QtWidgets.QDateTimeEdit.AmPmSection,
    msec=QtWidgets.QDateTimeEdit.MSecSection,
    second=QtWidgets.QDateTimeEdit.SecondSection,
    minute=QtWidgets.QDateTimeEdit.MinuteSection,
    hour=QtWidgets.QDateTimeEdit.HourSection,
    day=QtWidgets.QDateTimeEdit.DaySection,
    month=QtWidgets.QDateTimeEdit.MonthSection,
    year=QtWidgets.QDateTimeEdit.YearSection,
)

SectionsStr = Literal[
    "none",
    "am_pm",
    "msec",
    "second",
    "minute",
    "hour",
    "day",
    "month",
    "year",
]


QtWidgets.QDateTimeEdit.__bases__ = (widgets.AbstractSpinBox,)


class DateTimeEdit(QtWidgets.QDateTimeEdit):

    value_changed = core.Signal(datetime.datetime)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setCalendarPopup(True)
        self.dateTimeChanged.connect(self.datetime_changed)

    def __setstate__(self, state):
        self.setDateTime(state["datetime"])
        self.setEnabled(state.get("enabled", True))
        self.set_range(*state["range"])
        self.setDisplayFormat(state["display_format"])
        self.setToolTip(state.get("tool_tip", ""))
        self.setStatusTip(state.get("status_tip", ""))

    def __reduce__(self):
        return type(self), (), self.__getstate__()

    def datetime_changed(self, date):
        dt = self.get_datetime()
        self.value_changed.emit(dt)

    def serialize_fields(self):
        return dict(
            calendar_popup=self.calendarPopup(),
            datetime=self.get_datetime(),
            range=(self.min_datetime(), self.max_datetime()),
            display_format=self.displayFormat(),
        )

    def get_section_text(self, section: SectionsStr) -> str:
        if section not in SECTIONS:
            raise InvalidParamError(section, SECTIONS)
        return self.sectionText(SECTIONS[section])

    def get_current_section(self) -> SectionsStr:
        return SECTIONS.inverse[self.currentSection()]

    def set_current_section(self, section: SectionsStr):
        if section not in SECTIONS:
            raise InvalidParamError(section, SECTIONS)
        self.setCurrentSection(SECTIONS[section])

    def get_displayed_sections(self) -> List[SectionsStr]:
        return [k for k, v in SECTIONS.items() if v & self.displayedSections()]

    def set_range(
        self,
        lower: Union[QtCore.QDateTime, datetime.datetime],
        upper: Union[QtCore.QDateTime, datetime.datetime],
    ):
        self.setToolTip(f"{lower} <= x <= {upper}")
        self.setDateTimeRange(lower, upper)

    def set_format(self, fmt: str):
        self.setDisplayFormat(fmt)

    def get_value(self) -> datetime.datetime:
        return self.get_datetime()

    def set_value(self, value: datetime.datetime):
        self.setDateTime(value)

    def get_datetime(self) -> datetime.datetime:
        return to_datetime(self.dateTime())

    def min_datetime(self) -> datetime.datetime:
        return to_datetime(self.minimumDateTime())

    def max_datetime(self) -> datetime.datetime:
        return to_datetime(self.maximumDateTime())

    def min_date(self) -> datetime.date:
        return to_date(self.minimumDate())

    def max_date(self) -> datetime.date:
        return to_date(self.maximumDate())

    def get_date(self) -> datetime.date:
        return to_date(self.date())

    def min_time(self) -> datetime.time:
        return to_time(self.minimumTime())

    def max_time(self) -> datetime.time:
        return to_time(self.maximumTime())

    def get_time(self) -> datetime.time:
        return to_time(self.time())


if __name__ == "__main__":
    app = widgets.app()
    widget = DateTimeEdit()
    widget.show()
    app.main_loop()



from typing import List, Iterator

from qtpy import QtGui

from prettyqt import gui

QtGui.QTextBlockGroup.__bases__ = (gui.TextObject,)


class TextBlockGroup(QtGui.QTextBlockGroup):
    def __repr__(self):
        return f"{type(self).__name__}()"

    def __iter__(self) -> Iterator[gui.TextBlock]:
        return iter(gui.TextBlock(i) for i in self.blockList())

    def get_blocklist(self) -> List[gui.TextBlock]:
        return [gui.TextBlock(i) for i in self.blockList()]


if __name__ == "__main__":
    doc = gui.TextDocument()
    group = TextBlockGroup(doc)
    for textblock in group:
        pass



from qtpy import QtWidgets

from prettyqt import core, widgets


QtWidgets.QGraphicsEllipseItem.__bases__ = (widgets.AbstractGraphicsShapeItem,)


class GraphicsEllipseItem(QtWidgets.QGraphicsEllipseItem):
    def serialize_fields(self):
        return dict(
            rect=core.RectF(self.rect()),
            span_angle=self.spanAngle(),
            start_angle=self.startAngle(),
        )



import pathlib

from qtpy import uic


def create_ui(source, target):
    if isinstance(source, str):
        source = pathlib.Path(source)
    if isinstance(target, str):
        target = pathlib.Path(target)
    ui_files = [i for i in source.iterdir() if i.suffix == ".ui"]
    for ui_filename in ui_files:
        py_filename = ui_filename.with_suffix(".py")
        with (target / py_filename.name).open(mode="w") as fout:
            uic.compileUi(str(source / ui_filename), fout, from_imports=True)
            print(f"{py_filename.name} created in {target}.")
    # resources_path = target.parent.parent.parent / "resources" / "resources.qrc"
    # resources_output_path = target / "resources_rc.py"
    # os.system(f"pyrcc5.exe -o {resources_output_path} {resources_path}")
    # print(f"created {resources_output_path}.")
    #  pyrcc5.exe -o resources.py .\resources.qrc



from typing import Literal

from qtpy import QtLocation

from prettyqt.utils import bidict

try:  # PySide2 5.15 doesnt include these
    VISIBILITY = bidict(
        unspecified=QtLocation.QLocation.UnspecifiedVisibility,
        device=QtLocation.QLocation.DeviceVisibility,
        private=QtLocation.QLocation.PrivateVisibility,
        public=QtLocation.QLocation.PublicVisibility,
    )
except AttributeError:
    VISIBILITY = bidict()


VisibilityStr = Literal["unspecified", "device", "private", "public"]



from typing import Optional, Union, Iterator, Tuple, Pattern

from qtpy import QtGui, QtCore

from prettyqt import core, gui


QtGui.QSyntaxHighlighter.__bases__ = (core.Object,)


class SyntaxHighlighter(QtGui.QSyntaxHighlighter):

    RULES: list = []

    def __init__(
        self, parent: Optional[Union[QtCore.QObject, QtGui.QTextDocument]] = None
    ):
        super().__init__(parent)

    @classmethod
    def yield_rules(cls) -> Iterator[Tuple[Pattern, int, gui.TextCharFormat]]:
        for Rule in cls.RULES:
            if isinstance(Rule.compiled, list):
                for i in Rule.compiled:
                    yield (i, Rule.nth, Rule.fmt)
            else:
                yield (Rule.compiled, Rule.nth, Rule.fmt)

    def highlightBlock(self, text: str):
        """Apply syntax highlighting to the given block of text."""
        # Do other syntax formatting
        for expression, nth, fmt in self.yield_rules():
            for match in expression.finditer(text):
                span = match.span(nth)
                self.setFormat(span[0], span[1] - span[0], fmt)



from typing import Union, Literal

from qtpy import QtWidgets

from prettyqt import widgets
from prettyqt.utils import bidict, InvalidParamError

MODE = bidict(
    hex=QtWidgets.QLCDNumber.Hex,
    decimal=QtWidgets.QLCDNumber.Dec,
    octal=QtWidgets.QLCDNumber.Oct,
    binary=QtWidgets.QLCDNumber.Bin,
)

ModeStr = Literal["hex", "decimal", "octal", "binary"]

SEGMENT_STYLE = bidict(
    outline=QtWidgets.QLCDNumber.Outline,
    filled=QtWidgets.QLCDNumber.Filled,
    flat=QtWidgets.QLCDNumber.Flat,
)

SegmentStyleStr = Literal["outline", "filled", "flat"]

QtWidgets.QLCDNumber.__bases__ = (widgets.Frame,)


class LCDNumber(QtWidgets.QLCDNumber):
    def set_mode(self, mode: ModeStr):
        """Set the lcd mode.

        Args:
            mode: lcd mode to use

        Raises:
            InvalidParamError: lcd mode does not exist
        """
        if mode not in MODE:
            raise InvalidParamError(mode, MODE)
        self.setMode(MODE[mode])

    def get_mode(self) -> ModeStr:
        """Return current lcd mode.

        Returns:
            lcd mode
        """
        return MODE.inverse[self.mode()]

    def set_segment_style(self, mode: SegmentStyleStr):
        """Set the segment style.

        Args:
            mode: segment style to use

        Raises:
            InvalidParamError: segment style does not exist
        """
        if mode not in SEGMENT_STYLE:
            raise InvalidParamError(mode, SEGMENT_STYLE)
        self.setSegmentStyle(SEGMENT_STYLE[mode])

    def get_segment_style(self) -> SegmentStyleStr:
        """Return current segment style.

        Returns:
            segment style
        """
        return SEGMENT_STYLE.inverse[self.segmentStyle()]

    def set_value(self, value: Union[float, str]):
        self.display(value)

    def get_value(self) -> float:
        return self.value()


if __name__ == "__main__":
    app = widgets.app()
    widget = LCDNumber()
    widget.set_value(5555)
    widget.show()
    app.main_loop()



from qtpy import QtQuick

from prettyqt import gui, core, qml
from prettyqt.utils import bidict, InvalidParamError


FLAGS = bidict(
    clips_children_to_shape=QtQuick.QQuickItem.ItemClipsChildrenToShape,
    accepts_input_method=QtQuick.QQuickItem.ItemAcceptsInputMethod,
    is_focus_scope=QtQuick.QQuickItem.ItemIsFocusScope,
    has_contents=QtQuick.QQuickItem.ItemHasContents,
    accepts_drops=QtQuick.QQuickItem.ItemAcceptsDrops,
)

ITEM_CHANGE = bidict(
    child_added_change=QtQuick.QQuickItem.ItemChildAddedChange,
    child_removed_change=QtQuick.QQuickItem.ItemChildRemovedChange,
    item_scene_change=QtQuick.QQuickItem.ItemSceneChange,
    visible_has_changed=QtQuick.QQuickItem.ItemVisibleHasChanged,
    parent_has_changed=QtQuick.QQuickItem.ItemParentHasChanged,
    opacity_has_changed=QtQuick.QQuickItem.ItemOpacityHasChanged,
    active_focus_has_changed=QtQuick.QQuickItem.ItemActiveFocusHasChanged,
    rotation_has_changed=QtQuick.QQuickItem.ItemRotationHasChanged,
    pixel_ratio_has_changed=QtQuick.QQuickItem.ItemDevicePixelRatioHasChanged,
    anti_aliasing_has_changed=QtQuick.QQuickItem.ItemAntialiasingHasChanged,
    enabled_has_changed=QtQuick.QQuickItem.ItemEnabledHasChanged,
)

TRANSFORM_ORIGIN = bidict(
    top_left=QtQuick.QQuickItem.TopLeft,
    top=QtQuick.QQuickItem.Top,
    top_right=QtQuick.QQuickItem.TopRight,
    left=QtQuick.QQuickItem.Left,
    center=QtQuick.QQuickItem.Center,
    right=QtQuick.QQuickItem.Right,
    bottom_left=QtQuick.QQuickItem.BottomLeft,
    bottom=QtQuick.QQuickItem.Bottom,
    bottom_right=QtQuick.QQuickItem.BottomRight,
)


QtQuick.QQuickItem.__bases__ = (core.Object, qml.QmlParserStatus)


class QuickItem(QtQuick.QQuickItem):
    def get_children_rect(self) -> core.RectF:
        return core.RectF(self.childrenRect())

    def get_cursor(self) -> gui.Cursor:
        return gui.Cursor(self.cursor())

    def get_flags(self):
        pass

    def set_transform_origin(self, origin: str):
        """Set the origin point around which scale and rotation transform.

        The default is "center".

        Allowed values are "top_left", "top", "top_right", "left", "center", "right",
                           "bottom_left", "bottom", "bottom_right"

        Args:
            origin: transform origin to use

        Raises:
            InvalidParamError: transform origin does not exist
        """
        if origin not in TRANSFORM_ORIGIN:
            raise InvalidParamError(origin, TRANSFORM_ORIGIN)
        self.setTransformOrigin(TRANSFORM_ORIGIN[origin])

    def get_transform_origin(self) -> str:
        """Return the render type of text-like elements in Qt Quick.

        Possible values: "top_left", "top", "top_right", "left", "center", "right",
                         "bottom_left", "bottom", "bottom_right"

        Returns:
            transform origin
        """
        return TRANSFORM_ORIGIN.inverse[self.transformOrigin()]


if __name__ == "__main__":
    item = QuickItem()



# from qtpy import QtWebEngineWidgets
from typing import Literal

from qtpy import PYQT5, PYSIDE2

if PYQT5:
    from PyQt5 import QtWebEngineWidgets  # type: ignore
elif PYSIDE2:
    from PySide2 import QtWebEngineWidgets

from prettyqt import core
from prettyqt.utils import bidict, InvalidParamError


FONT_FAMILY = bidict(
    standard=QtWebEngineWidgets.QWebEngineSettings.StandardFont,
    fixed=QtWebEngineWidgets.QWebEngineSettings.FixedFont,
    serif=QtWebEngineWidgets.QWebEngineSettings.SerifFont,
    sans_serif=QtWebEngineWidgets.QWebEngineSettings.SansSerifFont,
    cursive=QtWebEngineWidgets.QWebEngineSettings.CursiveFont,
    fantasy=QtWebEngineWidgets.QWebEngineSettings.FantasyFont,
    pictograph=QtWebEngineWidgets.QWebEngineSettings.PictographFont,
)

FontFamilyStr = Literal[
    "standard", "fixed", "serif", "sans_serif", "cursive", "fantasy", "pictograph"
]

FONT_SIZE = bidict(
    minimum=QtWebEngineWidgets.QWebEngineSettings.MinimumFontSize,
    minimum_logical=QtWebEngineWidgets.QWebEngineSettings.MinimumLogicalFontSize,
    default=QtWebEngineWidgets.QWebEngineSettings.DefaultFontSize,
    default_fixed=QtWebEngineWidgets.QWebEngineSettings.DefaultFixedFontSize,
)

FontSizeStr = Literal["minimum", "minimum_logical", "default", "default_fixed"]

mod = QtWebEngineWidgets.QWebEngineSettings

UNKNOWN_URL_SCHEME_POLICY = bidict(
    disallow=mod.DisallowUnknownUrlSchemes,
    allow_from_user_interaction=mod.AllowUnknownUrlSchemesFromUserInteraction,
    allow_all=mod.AllowAllUnknownUrlSchemes,
)

UnknownUrlSchemePolicyStr = Literal[
    "disallow", "allow_from_user_interaction", "allow_all"
]

WEB_ATTRIBUTES = bidict(
    auto_load_images=mod.AutoLoadImages,
    javascript_enabled=mod.JavascriptEnabled,
    javascript_can_open_windows=mod.JavascriptCanOpenWindows,
    javascript_can_access_clipboard=mod.JavascriptCanAccessClipboard,
    links_included_in_focus_chain=mod.LinksIncludedInFocusChain,
    local_storage_enabled=mod.LocalStorageEnabled,
    local_content_can_access_remote_urls=mod.LocalContentCanAccessRemoteUrls,
    spatial_navigation_enabled=mod.SpatialNavigationEnabled,
    local_content_can_access_file_urls=mod.LocalContentCanAccessFileUrls,
    hyperlink_auditing_enabled=mod.HyperlinkAuditingEnabled,
    scroll_animator_enabled=mod.ScrollAnimatorEnabled,
    error_page_enabled=mod.ErrorPageEnabled,
    plugins_enabled=mod.PluginsEnabled,
    full_screen_support_enabled=mod.FullScreenSupportEnabled,
    screen_capture_enabled=mod.ScreenCaptureEnabled,
    web_gl_enabled=mod.WebGLEnabled,
    accelerated_2d_canvas_enabled=mod.Accelerated2dCanvasEnabled,
    auto_load_icons_for_page=mod.AutoLoadIconsForPage,
    touch_icons_enabled=mod.TouchIconsEnabled,
    focus_on_navigation_enabled=mod.FocusOnNavigationEnabled,
    print_element_backgrounds=mod.PrintElementBackgrounds,
    allow_running_insecure_content=mod.AllowRunningInsecureContent,
    allow_geolocation_on_insecure_origins=mod.AllowGeolocationOnInsecureOrigins,
    allow_window_activation_from_javascript=mod.AllowWindowActivationFromJavaScript,
    show_scrollbars=mod.ShowScrollBars,
    playback_requires_user_gesture=mod.PlaybackRequiresUserGesture,
    javascript_can_paste=mod.JavascriptCanPaste,
    web_rtc_public_interfaces_only=mod.WebRTCPublicInterfacesOnly,
    dns_prefetch_enabled=mod.DnsPrefetchEnabled,
)

if core.VersionNumber.get_qt_version() >= (5, 13, 0):
    WEB_ATTRIBUTES["pdf_viewer_enabled"] = mod.PdfViewerEnabled

WebAttributeStr = Literal[
    "auto_load_images",
    "javascript_enabled",
    "javascript_can_open_windows",
    "javascript_can_access_clipboard",
    "links_included_in_focus_chain",
    "local_storage_enabled",
    "local_content_can_access_remote_urls",
    "spatial_navigation_enabled",
    "local_content_can_access_file_urls",
    "hyperlink_auditing_enabled",
    "scroll_animator_enabled",
    "error_page_enabled",
    "plugins_enabled",
    "full_screen_support_enabled",
    "screen_capture_enabled",
    "web_gl_enabled",
    "accelerated_2d_canvas_enabled",
    "auto_load_icons_for_page",
    "touch_icons_enabled",
    "focus_on_navigation_enabled",
    "print_element_backgrounds",
    "allow_running_insecure_content",
    "allow_geolocation_on_insecure_origins",
    "allow_window_activation_from_javascript",
    "show_scrollbars",
    "playback_requires_user_gesture",
    "javascript_can_paste",
    "web_rtc_public_interfaces_only",
    "dns_prefetch_enabled",
    "pdf_viewer_enabled",
]


class WebEngineSettings:
    def __init__(self, item: QtWebEngineWidgets.QWebEngineSettings):
        self.item = item

    def __getattr__(self, val):
        return getattr(self.item, val)

    def __setitem__(self, index: WebAttributeStr, value: bool):
        self.item.setAttribute(WEB_ATTRIBUTES[index], value)

    def __getitem__(self, index: WebAttributeStr) -> bool:
        return self.item.testAttribute(WEB_ATTRIBUTES[index])

    def __delitem__(self, index: WebAttributeStr):
        return self.item.resetAttribute(WEB_ATTRIBUTES[index])

    def set_unknown_url_scheme_policy(self, policy: UnknownUrlSchemePolicyStr):
        """Set the unknown url scheme policy.

        Args:
            policy: unknown url scheme policy

        Raises:
            InvalidParamError: Policy does not exist
        """
        if policy not in UNKNOWN_URL_SCHEME_POLICY:
            raise InvalidParamError(policy, UNKNOWN_URL_SCHEME_POLICY)
        self.item.setUnknownUrlSchemePolicy(UNKNOWN_URL_SCHEME_POLICY[policy])

    def get_unknown_url_scheme_policy(self) -> UnknownUrlSchemePolicyStr:
        """Return current unknown url scheme policy.

        Returns:
            Unknown url scheme policy
        """
        return UNKNOWN_URL_SCHEME_POLICY.inverse[self.item.unknownUrlSchemePolicy()]

    def set_font_family(self, which: FontFamilyStr, family: str):
        """Set the actual font family to family for the specified generic family, which.

        Args:
            which: family to set
            family: generic family

        Raises:
            InvalidParamError: Font family does not exist
        """
        if which not in FONT_FAMILY:
            raise InvalidParamError(which, FONT_FAMILY)
        self.item.setFontFamily(FONT_FAMILY[which], family)

    def get_font_family(self, family: FontFamilyStr) -> str:
        """Return the actual font family for the specified generic font family.

        Args:
            family: generic font family

        Returns:
            Font family
        """
        return self.item.fontFamily(FONT_FAMILY[family])

    def set_font_size(self, typ: FontSizeStr, size: int):
        """Set the font size for type to size in pixels.

        Args:
            typ: font size type
            size: size in pixels

        Raises:
            InvalidParamError: Font size does not exist
        """
        if typ not in FONT_SIZE:
            raise InvalidParamError(typ, FONT_SIZE)
        self.item.setFontSize(FONT_SIZE[typ], size)

    def get_font_size(self, typ: FontSizeStr) -> int:
        """Return the default font size for type in pixels.

        Args:
            typ: font size type

        Returns:
            Font size
        """
        return self.item.fontSize(FONT_SIZE[typ])


if __name__ == "__main__":
    from prettyqt import widgets, webenginewidgets

    app = widgets.app()
    page = webenginewidgets.WebEnginePage()
    settings = page.get_settings()
    app.main_loop()



from typing import Literal

from qtpy import QtGui

from prettyqt import core
from prettyqt.utils import bidict, InvalidParamError

CACHE_MODE = bidict(none=QtGui.QMovie.CacheNone, all=QtGui.QMovie.CacheAll)

CacheModeStr = Literal["none", "all"]

MOVIE_STATE = bidict(
    not_running=QtGui.QMovie.NotRunning,
    paused=QtGui.QMovie.Paused,
    running=QtGui.QMovie.Running,
)

MovieStateStr = Literal["not_running", "paused", "running"]


QtGui.QMovie.__bases__ = (core.Object,)


class Movie(QtGui.QMovie):
    def __repr__(self):
        return f"{self.__class__.__name__}({self.fileName()!r}, {self.get_format()!r})"

    def serialize_fields(self):
        return dict(
            speed=self.speed(),
            cache_mode=self.get_cache_mode(),
            scaled_size=self.scaledSize(),
            background_color=self.backgroundColor(),
        )

    def set_cache_mode(self, mode: CacheModeStr):
        """Set cache mode.

        Args:
            mode: cache mode

        Raises:
            InvalidParamError: cache mode does not exist
        """
        if mode not in CACHE_MODE:
            raise InvalidParamError(mode, CACHE_MODE)
        self.setCacheMode(CACHE_MODE[mode])

    def get_cache_mode(self) -> CacheModeStr:
        """Get the current cache mode.

        Returns:
            cache mode
        """
        return CACHE_MODE.inverse[self.cacheMode()]

    def get_format(self) -> bytes:
        return bytes(self.format())


if __name__ == "__main__":
    from prettyqt import gui

    app = gui.app()
    movie = Movie()
    print(repr(movie))



from typing import Dict, Union

from qtpy import PYQT5, PYSIDE2, QtCore

if PYQT5:
    from PyQt5 import QtWebEngineCore  # type: ignore
elif PYSIDE2:
    from PySide2 import QtWebEngineCore

from prettyqt import core
from prettyqt.utils import bidict, InvalidParamError


METHODS = bidict(
    get=QtWebEngineCore.QWebEngineHttpRequest.Get,
    post=QtWebEngineCore.QWebEngineHttpRequest.Post,
)


class WebEngineHttpRequest(QtWebEngineCore.QWebEngineHttpRequest):
    def __repr__(self):
        return f"{type(self).__name__}({self.get_url()})"

    def set_headers(self, headers: Dict[str, str]):
        for k, v in headers.items():
            self.setHeader(k.encode(), v.encode())

    def get_headers(self) -> Dict[str, str]:
        return {bytes(h).decode(): bytes(self.header(h)).decode() for h in self.headers()}

    def set_url(self, url: Union[str, QtCore.QUrl]):
        url = core.Url(url)
        self.setUrl(url)

    def get_url(self) -> core.Url:
        return core.Url(self.url())

    def get_post_data(self) -> str:
        return bytes(self.postData()).decode()

    def set_method(self, method: str):
        """Set method this WebEngine request is using.

        Valid values for method: "post", "get"

        Args:
            method: method

        Raises:
            InvalidParamError: method does not exist
        """
        if method not in METHODS:
            raise InvalidParamError(method, METHODS)
        self.setMethod(METHODS[method])

    def get_method(self) -> str:
        """Get the method this WebEngine request is using.

        Possible values: "post", "get"

        Returns:
            method
        """
        return METHODS.inverse[self.method()]


if __name__ == "__main__":
    request = WebEngineHttpRequest()
    request.set_headers(dict(a="b"))
    print(request.get_headers())



from qtpy import QtGui

from prettyqt.utils import bidict

TYPES = bidict(
    variable=QtGui.QTextLength.VariableLength,
    fixed=QtGui.QTextLength.FixedLength,
    percentage=QtGui.QTextLength.PercentageLength,
)


class TextLength(QtGui.QTextLength):
    def __repr__(self):
        return f"{type(self).__name__}({self.type()}, {self.rawValue()})"

    def get_type(self) -> str:
        """Return type of this length object.

        Possible values: "variable", "fixed", "percentage"

        Returns:
            timer type
        """
        return TYPES.inverse[self.type()]


if __name__ == "__main__":
    length = TextLength()



from typing import Literal

from qtpy import QtWidgets, QtCore

from prettyqt import core, widgets
from prettyqt.utils import bidict, InvalidParamError


GESTURE_TYPE = bidict(
    tap=QtCore.Qt.TapGesture,
    tap_and_hold=QtCore.Qt.TapAndHoldGesture,
    pan=QtCore.Qt.PanGesture,
    pinch=QtCore.Qt.PinchGesture,
    swipe=QtCore.Qt.SwipeGesture,
    custom=QtCore.Qt.CustomGesture,
)

GestureTypeStr = Literal["tap", "tap_and_hold", "pan", "pinch", "swipe", "custom"]

GESTURE_STATE = bidict(
    none=0,  # QtCore.Qt.NoGesture,
    started=QtCore.Qt.GestureStarted,
    updated=QtCore.Qt.GestureUpdated,
    finished=QtCore.Qt.GestureFinished,
    canceled=QtCore.Qt.GestureCanceled,
)

GestureStateStr = Literal["none", "started", "updated", "finished", "canceled"]

GESTURE_CANCEL_POLICY = bidict(
    none=QtWidgets.QGesture.CancelNone,
    all_in_context=QtWidgets.QGesture.CancelAllInContext,
)

GestureCancelPolicyStr = Literal["none", "all_in_context"]

QtWidgets.QGesture.__bases__ = (core.Object,)


class Gesture(QtWidgets.QGesture):
    def get_state(self) -> GestureStateStr:
        """Return current state.

        Returns:
            state
        """
        return GESTURE_STATE.inverse[self.state()]

    def get_gesture_type(self) -> GestureTypeStr:
        """Return current gesture type.

        Returns:
            gesture type
        """
        return GESTURE_TYPE.inverse[self.gestureType()]

    def get_hot_spot(self) -> core.PointF:
        return core.PointF(self.hotSpot())

    def set_gesture_cancel_policy(self, policy: GestureCancelPolicyStr):
        """Set gesture cancel policy.

        Args:
            policy: gesture cancel policy to use

        Raises:
            InvalidParamError: gesture cancel policy does not exist
        """
        if policy not in GESTURE_CANCEL_POLICY:
            raise InvalidParamError(policy, GESTURE_CANCEL_POLICY)
        self.setGestureCancelPolicy(GESTURE_CANCEL_POLICY[policy])

    def get_gesture_cancel_policy(self) -> GestureCancelPolicyStr:
        """Return current gesture cancel policy.

        Returns:
            gesture cancel policy
        """
        return GESTURE_CANCEL_POLICY.inverse[self.gestureCancelPolicy()]


if __name__ == "__main__":
    app = widgets.app()
    gesture = Gesture()



from qtpy import QtCore


class Point(QtCore.QPoint):
    def __repr__(self):
        return f"{type(self).__name__}(x={self.x()}, y={self.y()})"

    def __getitem__(self, index) -> int:
        return (self.x(), self.y())[index]

    def __reduce__(self):
        return self.__class__, (self.x(), self.y())



from typing import Union, Literal

from qtpy import QtCore

from prettyqt import core
from prettyqt.utils import bidict, InvalidParamError


FLOAT_PRECISION = bidict(
    single=QtCore.QDataStream.SinglePrecision,
    double=QtCore.QDataStream.DoublePrecision,
)

FloatPrecisionStr = Literal["single", "double"]

BYTE_ORDER = bidict(
    big_endian=QtCore.QDataStream.BigEndian,
    little_endian=QtCore.QDataStream.LittleEndian,
)

ByteOrderStr = Literal["big_endian", "little_endian"]


class DataStream(QtCore.QDataStream):
    def set_byte_order(self, order: ByteOrderStr):
        """Set byte order.

        Args:
            order: byte order to use

        Raises:
            InvalidParamError: invalid order
        """
        if order not in BYTE_ORDER:
            raise InvalidParamError(order, BYTE_ORDER)
        self.setByteOrder(BYTE_ORDER[order])

    def get_byte_order(self) -> ByteOrderStr:
        """Return byte order.

        Returns:
            byte order
        """
        return BYTE_ORDER.inverse[self.byteOrder()]

    def set_float_precision(self, precision: FloatPrecisionStr):
        """Set floating point precision.

        Args:
            precision: floating point precision

        Raises:
            InvalidParamError: invalid precision
        """
        if precision not in FLOAT_PRECISION:
            raise InvalidParamError(precision, FLOAT_PRECISION)
        self.setFloatingPointPrecision(FLOAT_PRECISION[precision])

    def get_float_precision(self) -> FloatPrecisionStr:
        """Return floating point precision.

        Returns:
            floating point precision
        """
        return FLOAT_PRECISION.inverse[self.floatingPointPrecision()]

    @classmethod
    def create_bytearray(cls, data) -> QtCore.QByteArray:
        ba = QtCore.QByteArray()
        stream = cls(ba, core.IODevice.WriteOnly)
        stream << data
        return ba

    @classmethod
    def write_bytearray(cls, ba: Union[QtCore.QByteArray, bytes], write_to):
        if not isinstance(ba, QtCore.QByteArray):
            ba = QtCore.QByteArray(ba)
        stream = cls(ba, core.IODevice.ReadOnly)
        stream >> write_to

    @classmethod
    def copy_data(cls, source, dest):
        ba = cls.create_bytearray(source)
        cls.write_bytearray(ba, dest)



import contextlib
import pathlib
from typing import List, Mapping, Optional, Union, Dict, Any, Iterator, Tuple, Literal
import logging

from qtpy import QtCore

from prettyqt import core
from prettyqt.utils import bidict, InvalidParamError

logger = logging.getLogger(__name__)

FORMAT = bidict(native=QtCore.QSettings.NativeFormat, ini=QtCore.QSettings.IniFormat)

FormatStr = Literal["user", "system"]

SCOPE = bidict(user=QtCore.QSettings.UserScope, system=QtCore.QSettings.SystemScope)

ScopeStr = Literal["user", "system"]

QtCore.QSettings.__bases__ = (core.Object,)


class Settings(QtCore.QSettings):
    def __init__(self, *args, settings_id: Optional[str] = None):
        super().__init__(*args)
        self.settings_id = settings_id

    def __repr__(self):
        return f"{type(self).__name__}: {self.as_dict()}"

    def __contains__(self, key: str) -> bool:
        return self.contains(key)

    def __enter__(self):
        if self.settings_id:
            self.beginGroup(self.settings_id)
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        if self.settings_id:
            self.endGroup()

    def __getitem__(self, index: str):
        return self.get_value(index)

    def __setitem__(self, name: str, value):
        return self.set_value(name, value)

    def __delitem__(self, key: str):
        if not self.contains(key):
            raise KeyError(key)
        return self.remove(key)

    def __iter__(self) -> Iterator[Tuple[str, Any]]:
        return iter(self.items())

    def __len__(self) -> int:
        return len(self.allKeys())

    @classmethod
    def build_from_dict(cls, dct: Dict[str, Any]):
        settings = cls()
        for k, v in dct.items():
            settings.set_value(k, v)
        return settings

    def as_dict(self) -> Dict[str, Any]:
        return {k: v for k, v in self.items()}

    def set_value(self, key: str, value):
        if not self.applicationName():
            raise RuntimeError("no app name defined")
        self.setValue(key, dict(value=value))

    def set_values(self, dct: Dict[str, Any]):
        for k, v in dct.items():
            self.set_value(k, v)

    def get_value(self, key: str, default=None):
        if not self.contains(key):
            return default
        val = self.value(key)
        # this is for migration
        if not isinstance(val, dict) or "value" not in val:
            self.set_value(key, val)
            return val
        return val["value"]

    @classmethod
    def set_default_format(cls, fmt: FormatStr):
        """Set the default format.

        Args:
            fmt: the default format to use

        Raises:
            InvalidParamError: invalid format
        """
        if fmt not in FORMAT:
            raise InvalidParamError(fmt, FORMAT)
        cls.setDefaultFormat(FORMAT[fmt])

    @classmethod
    def get_default_format(cls) -> FormatStr:
        """Return default settings format.

        Returns:
            default settings format
        """
        return FORMAT.inverse[cls.defaultFormat()]

    def get_scope(self) -> ScopeStr:
        """Return scope.

        Returns:
            scope
        """
        return SCOPE.inverse[self.scope()]

    @classmethod
    def set_path(cls, fmt: FormatStr, scope: ScopeStr, path: Union[str, pathlib.Path]):
        """Set the path to the settings file.

        Args:
            fmt: the default format to use
            scope: the scope to use

        Raises:
            InvalidParamError: invalid format or scope
        """
        if fmt not in FORMAT:
            raise InvalidParamError(fmt, FORMAT)
        if scope not in SCOPE:
            raise InvalidParamError(scope, SCOPE)
        cls.setPath(FORMAT[fmt], SCOPE[scope], str(path))

    @contextlib.contextmanager
    def group(self, prefix: str):
        """Context manager for setting groups.

        Args:
            prefix: setting prefix for group
        """
        self.beginGroup(prefix)
        yield None
        self.endGroup()

    @contextlib.contextmanager
    def write_array(self, prefix: str, size: int = -1):
        """Context manager for writing arrays.

        Args:
            prefix: prefix for settings array
            size: size of settings array
        """
        self.beginWriteArray(prefix, size)
        yield None
        self.endArray()

    @contextlib.contextmanager
    def read_array(self, prefix: str):
        """Context manager for reading arrays.

        Args:
            prefix: prefix for settings array
        """
        self.beginReadArray(prefix)
        yield None
        self.endArray()

    # Dictionary interface

    def get(self, key: str, default: Any = None) -> Any:
        return self.get_value(key, default)

    def setdefault(self, key: str, default: Any = None) -> Any:
        if not self.contains(key):
            self.set_value(key, default)
            return default
        return self.get_value(key)

    def keys(self) -> List[str]:
        return self.allKeys()

    def values(self) -> Iterator[Any]:
        return (self.get_value(key) for key in self.allKeys())

    def items(self):
        return zip(self.keys(), self.values())

    def pop(self, key: str):
        if self.contains(key):
            return self.get_value(key)
        raise KeyError(key)

    def popitem(self) -> Tuple[str, Any]:
        key = self.keys()[0]
        return (key, self.get_value(key))

    def update(self, other: Mapping):
        for k, v in other.items():
            self.set_value(k, v)


def register_extensions(
    *exts: str,
    app_name: Optional[str] = None,
    app_path: Union[None, str, pathlib.Path] = None,
):
    logger.debug(f"assigning extensions {exts} to {app_name}")
    s = Settings("HKEY_CURRENT_USER\\SOFTWARE\\Classes", Settings.NativeFormat)
    if app_path is None:
        app_path = str(core.CoreApplication.get_application_file_path())
    if app_name is None:
        app_name = core.CoreApplication.applicationName()
    for ext in exts:
        s.setValue(f"{ext}/DefaultIcon/.", app_path)  # perhaps ,0 after app_path
        s.setValue(f"{ext}/.", app_name)
    s.setValue(f"{app_name}/shell/open/command/.", f"{app_path} %1")


if __name__ == "__main__":
    settings = Settings("1", "2")
    settings["1"] = True
    print(settings["1"])
    print(type(settings.get("1")))
    del settings["hallo"]



from typing import Literal

from qtpy import QtGui, QtCore

from prettyqt import core, gui
from prettyqt.utils import bidict

PATTERN = bidict(
    none=QtCore.Qt.NoBrush,
    solid=QtCore.Qt.SolidPattern,
    dense_1=QtCore.Qt.Dense1Pattern,
    dense_2=QtCore.Qt.Dense2Pattern,
    dense_3=QtCore.Qt.Dense3Pattern,
    dense_4=QtCore.Qt.Dense4Pattern,
    dense_5=QtCore.Qt.Dense5Pattern,
    dense_6=QtCore.Qt.Dense6Pattern,
    dense_7=QtCore.Qt.Dense7Pattern,
    horizontal=QtCore.Qt.HorPattern,
    vertical=QtCore.Qt.VerPattern,
    cross=QtCore.Qt.CrossPattern,
    backward_diagonal=QtCore.Qt.BDiagPattern,
    forward_diagonal=QtCore.Qt.FDiagPattern,
    crossing_diagonal=QtCore.Qt.DiagCrossPattern,
    linear_gradient=QtCore.Qt.LinearGradientPattern,
    conical_gradient=QtCore.Qt.ConicalGradientPattern,
    radial_gradient=QtCore.Qt.RadialGradientPattern,
    texture=QtCore.Qt.TexturePattern,
)

PatternStr = Literal[
    "none",
    "solid",
    "dense_1",
    "dense_2",
    "dense_3",
    "dense_4",
    "dense_5",
    "dense_6",
    "dense_7",
    "horizontal",
    "vertical",
    "cross",
    "backward_diagonal",
    "forward_diagonal",
    "crossing_diagonal",
    "linear_gradient",
    "conical_gradient",
    "radial_gradient",
    "texture",
]


class Brush(QtGui.QBrush):
    def __repr__(self):
        return f"{type(self).__name__}({self.get_color()!r}, {self.get_style()!r})"

    def __getstate__(self):
        return bytes(self)

    def __setstate__(self, ba):
        core.DataStream.write_bytearray(ba, self)

    def __reduce__(self):
        return type(self), (), self.__getstate__()

    def __bytes__(self):
        ba = core.DataStream.create_bytearray(self)
        return bytes(ba)

    def get_texture_image(self) -> gui.Image:
        return gui.Image(self.textureImage())

    def get_color(self) -> gui.Color:
        return gui.Color(self.color())

    def get_style(self) -> PatternStr:
        return PATTERN.inv[self.style()]


if __name__ == "__main__":
    b = Brush()
    print(repr(b))



from qtpy import QtWidgets

from prettyqt import widgets

QtWidgets.QSizeGrip.__bases__ = (widgets.Widget,)


class SizeGrip(QtWidgets.QSizeGrip):
    pass


if __name__ == "__main__":
    app = widgets.app()
    widget = SizeGrip()
    widget.show()
    app.main_loop()



import functools
import hashlib
from typing import Iterable, Optional, Union, List

from deprecated import deprecated
from qtpy import QtCore, QtWidgets

from prettyqt import core, widgets
from prettyqt.utils import bidict, InvalidParamError


ORIENTATIONS = bidict(horizontal=QtCore.Qt.Horizontal, vertical=QtCore.Qt.Vertical)

MODES = bidict(
    interactive=QtWidgets.QHeaderView.Interactive,
    fixed=QtWidgets.QHeaderView.Fixed,
    stretch=QtWidgets.QHeaderView.Stretch,
    resize_to_contents=QtWidgets.QHeaderView.ResizeToContents,
)


QtWidgets.QHeaderView.__bases__ = (widgets.AbstractItemView,)


class HeaderView(QtWidgets.QHeaderView):

    section_vis_changed = QtCore.Signal(int, bool)

    def __init__(
        self, orientation: Union[str, int], parent: Optional[QtWidgets.QWidget] = None
    ):
        if orientation in ORIENTATIONS:
            orientation = ORIENTATIONS[orientation]
        super().__init__(orientation, parent=parent)
        self.setSectionsMovable(True)
        self.setSectionsClickable(True)
        self._widget_name = parent.get_id() if parent is not None else ""

    def generate_header_id(self):
        # return f"{self._widget_name}.state"
        column_names = ",".join(self.get_section_labels())
        columns_hash = hashlib.md5(column_names.encode("utf-8")).hexdigest()
        return f"{type(self).__name__}_{columns_hash}.state"

    def save_state(
        self, settings: Optional[core.Settings] = None, key: Optional[str] = None
    ):
        settings = core.Settings() if settings is None else settings
        key = self.generate_header_id() if key is None else key
        settings.set_value(key, self.saveState())

    def load_state(
        self, settings: Optional[core.Settings] = None, key: Optional[str] = None
    ) -> bool:
        settings = core.Settings() if settings is None else settings
        key = self.generate_header_id() if key is None else key
        state = settings.get(key, None)
        if state is not None:
            self.restoreState(state)
            return True
        return False

    def resize_sections(self, mode: str):
        self.resizeSections(MODES[mode])

    @deprecated(reason="This method is deprecated, use set_resize_mode instead.")
    def resize_mode(self, mode: str, col: Optional[int] = None):
        self.set_resize_mode(mode, col)

    def set_resize_mode(self, mode: str, col: Optional[int] = None):
        if mode not in MODES:
            raise InvalidParamError(mode, MODES)
        if col is None:
            self.setSectionResizeMode(MODES[mode])
        else:
            self.setSectionResizeMode(col, MODES[mode])

    def get_section_labels(self) -> List[str]:
        model = self.model()
        return [
            model.headerData(i, QtCore.Qt.Horizontal, QtCore.Qt.DisplayRole)
            for i in range(self.count())
        ]

    def contextMenuEvent(self, event):
        """Context menu for our files tree."""
        menu = widgets.Menu(parent=self)
        actions = self.get_header_actions()
        menu.add_actions(actions)
        menu.exec_(self.mapToGlobal(event.pos()))

    def get_header_actions(self) -> List[widgets.Action]:
        actions = list()
        labels = self.get_section_labels()[1:]
        for i, header_label in enumerate(labels, start=1):
            val = not self.isSectionHidden(i)
            action = widgets.Action(text=header_label, checkable=True, checked=val)
            fn = functools.partial(self.set_section_hidden, i=i, hide=val)
            action.triggered.connect(fn)
            actions.append(action)
        return actions

    def set_section_hidden(self, i: int, hide: bool):
        self.section_vis_changed.emit(i, hide)
        self.setSectionHidden(i, hide)

    def set_sizes(self, sizes: Iterable[Optional[int]]):
        for i, size in enumerate(sizes):
            if size is not None:
                self.resizeSection(i, size)

    def set_default_section_size(self, size: Optional[int]):
        if size is None:
            self.resetDefaultSectionSize()
        else:
            self.setDefaultSectionSize(size)

    def stretch_last_section(self, stretch: bool = True):
        self.setStretchLastSection(stretch)


if __name__ == "__main__":
    app = widgets.app()
    header = HeaderView("horizontal")
    header.show()
    app.main_loop()
